=== modified file 'NEWS'
--- NEWS	2009-08-22 05:05:42 +0000
+++ NEWS	2009-08-26 03:20:32 +0000
@@ -36,6 +36,16 @@
   raised by newer versions of pycurl.
   (Vincent Ladeuil, #306264)
 
+* Fix a test failure on karmic by making a locale test more robust.
+  (Vincent Ladeuil, #413514)
+
+* Fix IndexError printing CannotBindAddress errors.
+  (Martin Pool, #286871)
+
+* Fix "Revision ... not present" errors when upgrading stacked branches,
+  or when doing fetches from a stacked source to a stacked target.
+  (Andrew Bennetts, #399140)
+
 Improvements
 ************
 
@@ -50,12 +60,36 @@
   ``bazaar.conf``.
   (Martin Pool, Robert Collins, #389328)
 
+* ``bzr push`` locally on windows will no longer give a locking error with
+  dirstate based formats. (Robert Collins)
+
+* ``bzr shelve`` and ``bzr unshelve`` now work on windows.
+  (Robert Collins, #305006)
+
+* Commit of specific files no longer prevents using the the iter_changes
+  codepath. On 2a repositories, commit of specific files should now be as
+  fast, or slightly faster, than a full commit. (Robert Collins)
+
+* The internal core code that handles specific file operations like
+  ``bzr st FILENAME`` or ``bzr commit FILENAME`` has been changed to
+  include the parent directories if they have altered, and when a
+  directory stops being a directory its children are always included. This
+  fixes a number of causes for ``InconsistentDelta`` errors, and permits
+  faster commit of specific paths. (Robert Collins, #347649)
+
 Documentation
 *************
 
+* New developer documentation for content filtering.
+  (Martin Pool)
+
 API Changes
 ***********
 
+* ``bzrlib.shelf_ui`` has had the ``from_args`` convenience methods of its
+  classes changed to manage lock lifetime of the trees they open in a way
+  consistent with reader-exclusive locks. (Robert Collins, #305006)
+
 Internals
 *********
 
@@ -282,7 +316,9 @@
 * Requests for unknown methods no longer cause the smart server to log
   lots of backtraces about ``UnknownSmartMethod``, ``do_chunk`` or
   ``do_end``.  (Andrew Bennetts, #338561)
-  
+
+* Shelve will not shelve the initial add of the tree root.  (Aaron Bentley)
+
 * Streaming from bzr servers where there is a chain of stacked branches
   (A stacked on B stacked on C) will now work. (Robert Collins, #406597)
 

=== modified file 'bzrlib/_dirstate_helpers_pyx.pyx'
--- bzrlib/_dirstate_helpers_pyx.pyx	2009-07-27 05:44:19 +0000
+++ bzrlib/_dirstate_helpers_pyx.pyx	2009-08-26 03:20:32 +0000
@@ -28,7 +28,7 @@
 
 from bzrlib import cache_utf8, errors, osutils
 from bzrlib.dirstate import DirState
-from bzrlib.osutils import pathjoin, splitpath
+from bzrlib.osutils import parent_directories, pathjoin, splitpath
 
 
 # This is the Windows equivalent of ENOTDIR
@@ -963,15 +963,21 @@
 
 cdef class ProcessEntryC:
 
+    cdef int doing_consistency_expansion
     cdef object old_dirname_to_file_id # dict
     cdef object new_dirname_to_file_id # dict
     cdef object last_source_parent
     cdef object last_target_parent
-    cdef object include_unchanged
+    cdef int include_unchanged
+    cdef int partial
     cdef object use_filesystem_for_exec
     cdef object utf8_decode
     cdef readonly object searched_specific_files
+    cdef readonly object searched_exact_paths
     cdef object search_specific_files
+    # The parents up to the root of the paths we are searching.
+    # After all normal paths are returned, these specific items are returned.
+    cdef object search_specific_file_parents
     cdef object state
     # Current iteration variables:
     cdef object current_root
@@ -989,31 +995,48 @@
     cdef object current_block_list
     cdef object current_dir_info
     cdef object current_dir_list
+    cdef object _pending_consistent_entries # list
     cdef int path_index
     cdef object root_dir_info
     cdef object bisect_left
     cdef object pathjoin
     cdef object fstat
+    # A set of the ids we've output when doing partial output.
+    cdef object seen_ids
     cdef object sha_file
 
     def __init__(self, include_unchanged, use_filesystem_for_exec,
         search_specific_files, state, source_index, target_index,
         want_unversioned, tree):
+        self.doing_consistency_expansion = 0
         self.old_dirname_to_file_id = {}
         self.new_dirname_to_file_id = {}
+        # Are we doing a partial iter_changes?
+        self.partial = set(['']).__ne__(search_specific_files)
         # Using a list so that we can access the values and change them in
         # nested scope. Each one is [path, file_id, entry]
         self.last_source_parent = [None, None]
         self.last_target_parent = [None, None]
-        self.include_unchanged = include_unchanged
+        if include_unchanged is None:
+            self.include_unchanged = False
+        else:
+            self.include_unchanged = int(include_unchanged)
         self.use_filesystem_for_exec = use_filesystem_for_exec
         self.utf8_decode = cache_utf8._utf8_decode
         # for all search_indexs in each path at or under each element of
-        # search_specific_files, if the detail is relocated: add the id, and add the
-        # relocated path as one to search if its not searched already. If the
-        # detail is not relocated, add the id.
+        # search_specific_files, if the detail is relocated: add the id, and
+        # add the relocated path as one to search if its not searched already.
+        # If the detail is not relocated, add the id.
         self.searched_specific_files = set()
+        # When we search exact paths without expanding downwards, we record
+        # that here.
+        self.searched_exact_paths = set()
         self.search_specific_files = search_specific_files
+        # The parents up to the root of the paths we are searching.
+        # After all normal paths are returned, these specific items are returned.
+        self.search_specific_file_parents = set()
+        # The ids we've sent out in the delta.
+        self.seen_ids = set()
         self.state = state
         self.current_root = None
         self.current_root_unicode = None
@@ -1035,26 +1058,30 @@
         self.current_block_pos = -1
         self.current_dir_info = None
         self.current_dir_list = None
+        self._pending_consistent_entries = []
         self.path_index = 0
         self.root_dir_info = None
         self.bisect_left = bisect.bisect_left
         self.pathjoin = osutils.pathjoin
         self.fstat = os.fstat
         self.sha_file = osutils.sha_file
+        if target_index != 0:
+            # A lot of code in here depends on target_index == 0
+            raise errors.BzrError('unsupported target index')
 
     cdef _process_entry(self, entry, path_info):
         """Compare an entry and real disk to generate delta information.
 
         :param path_info: top_relpath, basename, kind, lstat, abspath for
-            the path of entry. If None, then the path is considered absent.
-            (Perhaps we should pass in a concrete entry for this ?)
+            the path of entry. If None, then the path is considered absent in 
+            the target (Perhaps we should pass in a concrete entry for this ?)
             Basename is returned as a utf8 string because we expect this
             tuple will be ignored, and don't want to take the time to
             decode.
         :return: (iter_changes_result, changed). If the entry has not been
             handled then changed is None. Otherwise it is False if no content
-            or metadata changes have occured, and None if any content or
-            metadata change has occured. If self.include_unchanged is True then
+            or metadata changes have occured, and True if any content or
+            metadata change has occurred. If self.include_unchanged is True then
             if changed is not None, iter_changes_result will always be a result
             tuple. Otherwise, iter_changes_result is None unless changed is
             True.
@@ -1099,9 +1126,12 @@
             else:
                 # add the source to the search path to find any children it
                 # has.  TODO ? : only add if it is a container ?
-                if not osutils.is_inside_any(self.searched_specific_files,
-                                             source_details[1]):
+                if (not self.doing_consistency_expansion and 
+                    not osutils.is_inside_any(self.searched_specific_files,
+                                             source_details[1])):
                     self.search_specific_files.add(source_details[1])
+                    # expanding from a user requested path, parent expansion
+                    # for delta consistency happens later.
                 # generate the old path; this is needed for stating later
                 # as well.
                 old_path = source_details[1]
@@ -1180,7 +1210,8 @@
                     file_id = entry[0][2]
                 self.old_dirname_to_file_id[old_path] = file_id
             # parent id is the entry for the path in the target tree
-            if old_dirname == self.last_source_parent[0]:
+            if old_basename and old_dirname == self.last_source_parent[0]:
+                # use a cached hit for non-root source entries.
                 source_parent_id = self.last_source_parent[1]
             else:
                 try:
@@ -1196,7 +1227,8 @@
                     self.last_source_parent[0] = old_dirname
                     self.last_source_parent[1] = source_parent_id
             new_dirname = entry[0][0]
-            if new_dirname == self.last_target_parent[0]:
+            if entry[0][1] and new_dirname == self.last_target_parent[0]:
+                # use a cached hit for non-root target entries.
                 target_parent_id = self.last_target_parent[1]
             else:
                 try:
@@ -1313,8 +1345,13 @@
             # a renamed parent. TODO: handle this efficiently. Its not
             # common case to rename dirs though, so a correct but slow
             # implementation will do.
-            if not osutils.is_inside_any(self.searched_specific_files, target_details[1]):
+            if (not self.doing_consistency_expansion and 
+                not osutils.is_inside_any(self.searched_specific_files,
+                    target_details[1])):
                 self.search_specific_files.add(target_details[1])
+                # We don't expand the specific files parents list here as
+                # the path is absent in target and won't create a delta with
+                # missing parent.
         elif ((source_minikind == c'r' or source_minikind == c'a') and
               (target_minikind == c'r' or target_minikind == c'a')):
             # neither of the selected trees contain this path,
@@ -1334,6 +1371,25 @@
     def iter_changes(self):
         return self
 
+    cdef void _gather_result_for_consistency(self, result):
+        """Check a result we will yield to make sure we are consistent later.
+        
+        This gathers result's parents into a set to output later.
+
+        :param result: A result tuple.
+        """
+        if not self.partial or not result[0]:
+            return
+        self.seen_ids.add(result[0])
+        new_path = result[1][1]
+        if new_path:
+            # Not the root and not a delete: queue up the parents of the path.
+            self.search_specific_file_parents.update(
+                osutils.parent_directories(new_path.encode('utf8')))
+            # Add the root directory which parent_directories does not
+            # provide.
+            self.search_specific_file_parents.add('')
+
     cdef void _update_current_block(self):
         if (self.block_index < len(self.state._dirblocks) and
             osutils.is_inside(self.current_root, self.state._dirblocks[self.block_index][0])):
@@ -1406,8 +1462,11 @@
             entry = self.root_entries[self.root_entries_pos]
             self.root_entries_pos = self.root_entries_pos + 1
             result, changed = self._process_entry(entry, self.root_dir_info)
-            if changed is not None and changed or self.include_unchanged:
-                return result
+            if changed is not None:
+                if changed:
+                    self._gather_result_for_consistency(result)
+                if changed or self.include_unchanged:
+                    return result
         # Have we finished the prior root, or never started one ?
         if self.current_root is None:
             # TODO: the pending list should be lexically sorted?  the
@@ -1416,12 +1475,12 @@
                 self.current_root = self.search_specific_files.pop()
             except KeyError:
                 raise StopIteration()
-            self.current_root_unicode = self.current_root.decode('utf8')
             self.searched_specific_files.add(self.current_root)
             # process the entries for this containing directory: the rest will be
             # found by their parents recursively.
             self.root_entries = self.state._entries_for_path(self.current_root)
             self.root_entries_len = len(self.root_entries)
+            self.current_root_unicode = self.current_root.decode('utf8')
             self.root_abspath = self.tree.abspath(self.current_root_unicode)
             try:
                 root_stat = os.lstat(self.root_abspath)
@@ -1458,6 +1517,8 @@
                 result, changed = self._process_entry(entry, self.root_dir_info)
                 if changed is not None:
                     path_handled = -1
+                    if changed:
+                        self._gather_result_for_consistency(result)
                     if changed or self.include_unchanged:
                         return result
             # handle unversioned specified paths:
@@ -1476,7 +1537,8 @@
                       )
             # If we reach here, the outer flow continues, which enters into the
             # per-root setup logic.
-        if self.current_dir_info is None and self.current_block is None:
+        if (self.current_dir_info is None and self.current_block is None and not
+            self.doing_consistency_expansion):
             # setup iteration of this root:
             self.current_dir_list = None
             if self.root_dir_info and self.root_dir_info[2] == 'tree-reference':
@@ -1606,6 +1668,8 @@
                         # advance the entry only, after processing.
                         result, changed = self._process_entry(current_entry, None)
                         if changed is not None:
+                            if changed:
+                                self._gather_result_for_consistency(result)
                             if changed or self.include_unchanged:
                                 return result
                     self.block_index = self.block_index + 1
@@ -1618,6 +1682,15 @@
             # More supplied paths to process
             self.current_root = None
             return self._iter_next()
+        # Start expanding more conservatively, adding paths the user may not
+        # have intended but required for consistent deltas.
+        self.doing_consistency_expansion = 1
+        if not self._pending_consistent_entries:
+            self._pending_consistent_entries = self._next_consistent_entries()
+        while self._pending_consistent_entries:
+            result, changed = self._pending_consistent_entries.pop()
+            if changed is not None:
+                return result
         raise StopIteration()
 
     cdef object _maybe_tree_ref(self, current_path_info):
@@ -1705,6 +1778,8 @@
                             current_path_info)
                         if changed is not None:
                             path_handled = -1
+                            if not changed and not self.include_unchanged:
+                                changed = None
                 # >- loop control starts here:
                 # >- entry
                 if advance_entry and current_entry is not None:
@@ -1726,7 +1801,7 @@
                             except UnicodeDecodeError:
                                 raise errors.BadFilenameEncoding(
                                     current_path_info[0], osutils._fs_enc)
-                            if result is not None:
+                            if changed is not None:
                                 raise AssertionError(
                                     "result is not None: %r" % result)
                             result = (None,
@@ -1737,6 +1812,7 @@
                                 (None, self.utf8_decode(current_path_info[1])[0]),
                                 (None, current_path_info[2]),
                                 (None, new_executable))
+                            changed = True
                         # dont descend into this unversioned path if it is
                         # a dir
                         if current_path_info[2] in ('directory'):
@@ -1755,9 +1831,12 @@
                                 current_path_info)
                     else:
                         current_path_info = None
-                if result is not None:
+                if changed is not None:
                     # Found a result on this pass, yield it
-                    return result
+                    if changed:
+                        self._gather_result_for_consistency(result)
+                    if changed or self.include_unchanged:
+                        return result
             if self.current_block is not None:
                 self.block_index = self.block_index + 1
                 self._update_current_block()
@@ -1769,3 +1848,123 @@
                     self.current_dir_list = self.current_dir_info[1]
                 except StopIteration:
                     self.current_dir_info = None
+
+    cdef object _next_consistent_entries(self):
+        """Grabs the next specific file parent case to consider.
+        
+        :return: A list of the results, each of which is as for _process_entry.
+        """
+        results = []
+        while self.search_specific_file_parents:
+            # Process the parent directories for the paths we were iterating.
+            # Even in extremely large trees this should be modest, so currently
+            # no attempt is made to optimise.
+            path_utf8 = self.search_specific_file_parents.pop()
+            if path_utf8 in self.searched_exact_paths:
+                # We've examined this path.
+                continue
+            if osutils.is_inside_any(self.searched_specific_files, path_utf8):
+                # We've examined this path.
+                continue
+            path_entries = self.state._entries_for_path(path_utf8)
+            # We need either one or two entries. If the path in
+            # self.target_index has moved (so the entry in source_index is in
+            # 'ar') then we need to also look for the entry for this path in
+            # self.source_index, to output the appropriate delete-or-rename.
+            selected_entries = []
+            found_item = False
+            for candidate_entry in path_entries:
+                # Find entries present in target at this path:
+                if candidate_entry[1][self.target_index][0] not in 'ar':
+                    found_item = True
+                    selected_entries.append(candidate_entry)
+                # Find entries present in source at this path:
+                elif (self.source_index is not None and
+                    candidate_entry[1][self.source_index][0] not in 'ar'):
+                    found_item = True
+                    if candidate_entry[1][self.target_index][0] == 'a':
+                        # Deleted, emit it here.
+                        selected_entries.append(candidate_entry)
+                    else:
+                        # renamed, emit it when we process the directory it
+                        # ended up at.
+                        self.search_specific_file_parents.add(
+                            candidate_entry[1][self.target_index][1])
+            if not found_item:
+                raise AssertionError(
+                    "Missing entry for specific path parent %r, %r" % (
+                    path_utf8, path_entries))
+            path_info = self._path_info(path_utf8, path_utf8.decode('utf8'))
+            for entry in selected_entries:
+                if entry[0][2] in self.seen_ids:
+                    continue
+                result, changed = self._process_entry(entry, path_info)
+                if changed is None:
+                    raise AssertionError(
+                        "Got entry<->path mismatch for specific path "
+                        "%r entry %r path_info %r " % (
+                        path_utf8, entry, path_info))
+                # Only include changes - we're outside the users requested
+                # expansion.
+                if changed:
+                    self._gather_result_for_consistency(result)
+                    if (result[6][0] == 'directory' and
+                        result[6][1] != 'directory'):
+                        # This stopped being a directory, the old children have
+                        # to be included.
+                        if entry[1][self.source_index][0] == 'r':
+                            # renamed, take the source path
+                            entry_path_utf8 = entry[1][self.source_index][1]
+                        else:
+                            entry_path_utf8 = path_utf8
+                        initial_key = (entry_path_utf8, '', '')
+                        block_index, _ = self.state._find_block_index_from_key(
+                            initial_key)
+                        if block_index == 0:
+                            # The children of the root are in block index 1.
+                            block_index = block_index + 1
+                        current_block = None
+                        if block_index < len(self.state._dirblocks):
+                            current_block = self.state._dirblocks[block_index]
+                            if not osutils.is_inside(
+                                entry_path_utf8, current_block[0]):
+                                # No entries for this directory at all.
+                                current_block = None
+                        if current_block is not None:
+                            for entry in current_block[1]:
+                                if entry[1][self.source_index][0] in 'ar':
+                                    # Not in the source tree, so doesn't have to be
+                                    # included.
+                                    continue
+                                # Path of the entry itself.
+                                self.search_specific_file_parents.add(
+                                    self.pathjoin(*entry[0][:2]))
+                if changed or self.include_unchanged:
+                    results.append((result, changed))
+            self.searched_exact_paths.add(path_utf8)
+        return results
+
+    cdef object _path_info(self, utf8_path, unicode_path):
+        """Generate path_info for unicode_path.
+
+        :return: None if unicode_path does not exist, or a path_info tuple.
+        """
+        abspath = self.tree.abspath(unicode_path)
+        try:
+            stat = os.lstat(abspath)
+        except OSError, e:
+            if e.errno == errno.ENOENT:
+                # the path does not exist.
+                return None
+            else:
+                raise
+        utf8_basename = utf8_path.rsplit('/', 1)[-1]
+        dir_info = (utf8_path, utf8_basename,
+            osutils.file_kind_from_stat_mode(stat.st_mode), stat,
+            abspath)
+        if dir_info[2] == 'directory':
+            if self.tree._directory_is_tree_reference(
+                unicode_path):
+                self.root_dir_info = self.root_dir_info[:2] + \
+                    ('tree-reference',) + self.root_dir_info[3:]
+        return dir_info

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2009-08-11 18:00:37 +0000
+++ bzrlib/builtins.py	2009-08-26 03:20:32 +0000
@@ -120,6 +120,15 @@
 
 
 def _get_one_revision_tree(command_name, revisions, branch=None, tree=None):
+    """Get a revision tree. Not suitable for commands that change the tree.
+    
+    Specifically, the basis tree in dirstate trees is coupled to the dirstate
+    and doing a commit/uncommit/pull will at best fail due to changing the
+    basis revision data.
+
+    If tree is passed in, it should be already locked, for lifetime management
+    of the trees internal cached state.
+    """
     if branch is None:
         branch = tree.branch
     if revisions is None:
@@ -3029,6 +3038,10 @@
                 raise errors.BzrCommandError("empty commit message specified")
             return my_message
 
+        # The API permits a commit with a filter of [] to mean 'select nothing'
+        # but the command line should not do that.
+        if not selected_list:
+            selected_list = None
         try:
             tree.commit(message_callback=get_message,
                         specific_files=selected_list,
@@ -5631,8 +5644,12 @@
         if writer is None:
             writer = bzrlib.option.diff_writer_registry.get()
         try:
-            Shelver.from_args(writer(sys.stdout), revision, all, file_list,
-                              message, destroy=destroy).run()
+            shelver = Shelver.from_args(writer(sys.stdout), revision, all,
+                file_list, message, destroy=destroy)
+            try:
+                shelver.run()
+            finally:
+                shelver.work_tree.unlock()
         except errors.UserAbort:
             return 0
 
@@ -5677,7 +5694,11 @@
 
     def run(self, shelf_id=None, action='apply'):
         from bzrlib.shelf_ui import Unshelver
-        Unshelver.from_args(shelf_id, action).run()
+        unshelver = Unshelver.from_args(shelf_id, action)
+        try:
+            unshelver.run()
+        finally:
+            unshelver.tree.unlock()
 
 
 class cmd_clean_tree(Command):

=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2009-08-18 05:18:52 +0000
+++ bzrlib/bzrdir.py	2009-08-21 02:10:06 +0000
@@ -129,8 +129,16 @@
         return True
 
     def check_conversion_target(self, target_format):
+        """Check that a bzrdir as a whole can be converted to a new format."""
+        # The only current restriction is that the repository content can be 
+        # fetched compatibly with the target.
         target_repo_format = target_format.repository_format
-        self.open_repository()._format.check_conversion_target(target_repo_format)
+        try:
+            self.open_repository()._format.check_conversion_target(
+                target_repo_format)
+        except errors.NoRepositoryPresent:
+            # No repo, no problem.
+            pass
 
     @staticmethod
     def _check_supported(format, allow_unsupported,

=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2009-07-15 05:54:37 +0000
+++ bzrlib/commit.py	2009-08-26 03:20:32 +0000
@@ -209,7 +209,8 @@
         :param timestamp: if not None, seconds-since-epoch for a
             postdated/predated commit.
 
-        :param specific_files: If true, commit only those files.
+        :param specific_files: If not None, commit only those files. An empty
+            list means 'commit no files'.
 
         :param rev_id: If set, use this as the new revision id.
             Useful for test or import commands that need to tightly
@@ -264,6 +265,8 @@
         self.master_locked = False
         self.recursive = recursive
         self.rev_id = None
+        # self.specific_files is None to indicate no filter, or any iterable to
+        # indicate a filter - [] means no files at all, as per iter_changes.
         if specific_files is not None:
             self.specific_files = sorted(
                 minimum_path_selection(specific_files))
@@ -285,7 +288,6 @@
         # the command line parameters, and the repository has fast delta
         # generation. See bug 347649.
         self.use_record_iter_changes = (
-            not self.specific_files and
             not self.exclude and 
             not self.branch.repository._format.supports_tree_reference and
             (self.branch.repository._format.fast_deltas or
@@ -333,7 +335,7 @@
             self._gather_parents()
             # After a merge, a selected file commit is not supported.
             # See 'bzr help merge' for an explanation as to why.
-            if len(self.parents) > 1 and self.specific_files:
+            if len(self.parents) > 1 and self.specific_files is not None:
                 raise errors.CannotCommitSelectedFileMerge(self.specific_files)
             # Excludes are a form of selected file commit.
             if len(self.parents) > 1 and self.exclude:
@@ -619,12 +621,13 @@
         """Update the commit builder with the data about what has changed.
         """
         exclude = self.exclude
-        specific_files = self.specific_files or []
+        specific_files = self.specific_files
         mutter("Selecting files for commit with filter %s", specific_files)
 
         self._check_strict()
         if self.use_record_iter_changes:
-            iter_changes = self.work_tree.iter_changes(self.basis_tree)
+            iter_changes = self.work_tree.iter_changes(self.basis_tree,
+                specific_files=specific_files)
             iter_changes = self._filter_iter_changes(iter_changes)
             for file_id, path, fs_hash in self.builder.record_iter_changes(
                 self.work_tree, self.basis_revid, iter_changes):

=== modified file 'bzrlib/dirstate.py'
--- bzrlib/dirstate.py	2009-07-27 05:44:19 +0000
+++ bzrlib/dirstate.py	2009-08-26 03:20:32 +0000
@@ -3166,15 +3166,18 @@
 
     __slots__ = ["old_dirname_to_file_id", "new_dirname_to_file_id",
         "last_source_parent", "last_target_parent", "include_unchanged",
-        "use_filesystem_for_exec", "utf8_decode", "searched_specific_files",
-        "search_specific_files", "state", "source_index", "target_index",
-        "want_unversioned", "tree"]
+        "partial", "use_filesystem_for_exec", "utf8_decode",
+        "searched_specific_files", "search_specific_files",
+        "searched_exact_paths", "search_specific_file_parents", "seen_ids",
+        "state", "source_index", "target_index", "want_unversioned", "tree"]
 
     def __init__(self, include_unchanged, use_filesystem_for_exec,
         search_specific_files, state, source_index, target_index,
         want_unversioned, tree):
         self.old_dirname_to_file_id = {}
         self.new_dirname_to_file_id = {}
+        # Are we doing a partial iter_changes?
+        self.partial = search_specific_files != set([''])
         # Using a list so that we can access the values and change them in
         # nested scope. Each one is [path, file_id, entry]
         self.last_source_parent = [None, None]
@@ -3183,14 +3186,25 @@
         self.use_filesystem_for_exec = use_filesystem_for_exec
         self.utf8_decode = cache_utf8._utf8_decode
         # for all search_indexs in each path at or under each element of
-        # search_specific_files, if the detail is relocated: add the id, and add the
-        # relocated path as one to search if its not searched already. If the
-        # detail is not relocated, add the id.
+        # search_specific_files, if the detail is relocated: add the id, and
+        # add the relocated path as one to search if its not searched already.
+        # If the detail is not relocated, add the id.
         self.searched_specific_files = set()
+        # When we search exact paths without expanding downwards, we record
+        # that here.
+        self.searched_exact_paths = set()
         self.search_specific_files = search_specific_files
+        # The parents up to the root of the paths we are searching.
+        # After all normal paths are returned, these specific items are returned.
+        self.search_specific_file_parents = set()
+        # The ids we've sent out in the delta.
+        self.seen_ids = set()
         self.state = state
         self.source_index = source_index
         self.target_index = target_index
+        if target_index != 0:
+            # A lot of code in here depends on target_index == 0
+            raise errors.BzrError('unsupported target index')
         self.want_unversioned = want_unversioned
         self.tree = tree
 
@@ -3198,15 +3212,15 @@
         """Compare an entry and real disk to generate delta information.
 
         :param path_info: top_relpath, basename, kind, lstat, abspath for
-            the path of entry. If None, then the path is considered absent.
-            (Perhaps we should pass in a concrete entry for this ?)
+            the path of entry. If None, then the path is considered absent in 
+            the target (Perhaps we should pass in a concrete entry for this ?)
             Basename is returned as a utf8 string because we expect this
             tuple will be ignored, and don't want to take the time to
             decode.
         :return: (iter_changes_result, changed). If the entry has not been
             handled then changed is None. Otherwise it is False if no content
-            or metadata changes have occured, and None if any content or
-            metadata change has occured. If self.include_unchanged is True then
+            or metadata changes have occurred, and True if any content or
+            metadata change has occurred. If self.include_unchanged is True then
             if changed is not None, iter_changes_result will always be a result
             tuple. Otherwise, iter_changes_result is None unless changed is
             True.
@@ -3463,6 +3477,25 @@
     def __iter__(self):
         return self
 
+    def _gather_result_for_consistency(self, result):
+        """Check a result we will yield to make sure we are consistent later.
+        
+        This gathers result's parents into a set to output later.
+
+        :param result: A result tuple.
+        """
+        if not self.partial or not result[0]:
+            return
+        self.seen_ids.add(result[0])
+        new_path = result[1][1]
+        if new_path:
+            # Not the root and not a delete: queue up the parents of the path.
+            self.search_specific_file_parents.update(
+                osutils.parent_directories(new_path.encode('utf8')))
+            # Add the root directory which parent_directories does not
+            # provide.
+            self.search_specific_file_parents.add('')
+
     def iter_changes(self):
         """Iterate over the changes."""
         utf8_decode = cache_utf8._utf8_decode
@@ -3546,6 +3579,8 @@
                 result, changed = _process_entry(entry, root_dir_info)
                 if changed is not None:
                     path_handled = True
+                    if changed:
+                        self._gather_result_for_consistency(result)
                     if changed or self.include_unchanged:
                         yield result
             if self.want_unversioned and not path_handled and root_dir_info:
@@ -3664,6 +3699,8 @@
                             # advance the entry only, after processing.
                             result, changed = _process_entry(current_entry, None)
                             if changed is not None:
+                                if changed:
+                                    self._gather_result_for_consistency(result)
                                 if changed or self.include_unchanged:
                                     yield result
                         block_index +=1
@@ -3702,6 +3739,8 @@
                         # no path is fine: the per entry code will handle it.
                         result, changed = _process_entry(current_entry, current_path_info)
                         if changed is not None:
+                            if changed:
+                                self._gather_result_for_consistency(result)
                             if changed or self.include_unchanged:
                                 yield result
                     elif (current_entry[0][1] != current_path_info[1]
@@ -3723,6 +3762,8 @@
                             # advance the entry only, after processing.
                             result, changed = _process_entry(current_entry, None)
                             if changed is not None:
+                                if changed:
+                                    self._gather_result_for_consistency(result)
                                 if changed or self.include_unchanged:
                                     yield result
                             advance_path = False
@@ -3730,6 +3771,8 @@
                         result, changed = _process_entry(current_entry, current_path_info)
                         if changed is not None:
                             path_handled = True
+                            if changed:
+                                self._gather_result_for_consistency(result)
                             if changed or self.include_unchanged:
                                 yield result
                     if advance_entry and current_entry is not None:
@@ -3795,6 +3838,124 @@
                         current_dir_info = dir_iterator.next()
                     except StopIteration:
                         current_dir_info = None
+        for result in self._iter_specific_file_parents():
+            yield result
+
+    def _iter_specific_file_parents(self):
+        """Iter over the specific file parents."""
+        while self.search_specific_file_parents:
+            # Process the parent directories for the paths we were iterating.
+            # Even in extremely large trees this should be modest, so currently
+            # no attempt is made to optimise.
+            path_utf8 = self.search_specific_file_parents.pop()
+            if osutils.is_inside_any(self.searched_specific_files, path_utf8):
+                # We've examined this path.
+                continue
+            if path_utf8 in self.searched_exact_paths:
+                # We've examined this path.
+                continue
+            path_entries = self.state._entries_for_path(path_utf8)
+            # We need either one or two entries. If the path in
+            # self.target_index has moved (so the entry in source_index is in
+            # 'ar') then we need to also look for the entry for this path in
+            # self.source_index, to output the appropriate delete-or-rename.
+            selected_entries = []
+            found_item = False
+            for candidate_entry in path_entries:
+                # Find entries present in target at this path:
+                if candidate_entry[1][self.target_index][0] not in 'ar':
+                    found_item = True
+                    selected_entries.append(candidate_entry)
+                # Find entries present in source at this path:
+                elif (self.source_index is not None and
+                    candidate_entry[1][self.source_index][0] not in 'ar'):
+                    found_item = True
+                    if candidate_entry[1][self.target_index][0] == 'a':
+                        # Deleted, emit it here.
+                        selected_entries.append(candidate_entry)
+                    else:
+                        # renamed, emit it when we process the directory it
+                        # ended up at.
+                        self.search_specific_file_parents.add(
+                            candidate_entry[1][self.target_index][1])
+            if not found_item:
+                raise AssertionError(
+                    "Missing entry for specific path parent %r, %r" % (
+                    path_utf8, path_entries))
+            path_info = self._path_info(path_utf8, path_utf8.decode('utf8'))
+            for entry in selected_entries:
+                if entry[0][2] in self.seen_ids:
+                    continue
+                result, changed = self._process_entry(entry, path_info)
+                if changed is None:
+                    raise AssertionError(
+                        "Got entry<->path mismatch for specific path "
+                        "%r entry %r path_info %r " % (
+                        path_utf8, entry, path_info))
+                # Only include changes - we're outside the users requested
+                # expansion.
+                if changed:
+                    self._gather_result_for_consistency(result)
+                    if (result[6][0] == 'directory' and
+                        result[6][1] != 'directory'):
+                        # This stopped being a directory, the old children have
+                        # to be included.
+                        if entry[1][self.source_index][0] == 'r':
+                            # renamed, take the source path
+                            entry_path_utf8 = entry[1][self.source_index][1]
+                        else:
+                            entry_path_utf8 = path_utf8
+                        initial_key = (entry_path_utf8, '', '')
+                        block_index, _ = self.state._find_block_index_from_key(
+                            initial_key)
+                        if block_index == 0:
+                            # The children of the root are in block index 1.
+                            block_index +=1
+                        current_block = None
+                        if block_index < len(self.state._dirblocks):
+                            current_block = self.state._dirblocks[block_index]
+                            if not osutils.is_inside(
+                                entry_path_utf8, current_block[0]):
+                                # No entries for this directory at all.
+                                current_block = None
+                        if current_block is not None:
+                            for entry in current_block[1]:
+                                if entry[1][self.source_index][0] in 'ar':
+                                    # Not in the source tree, so doesn't have to be
+                                    # included.
+                                    continue
+                                # Path of the entry itself.
+
+                                self.search_specific_file_parents.add(
+                                    osutils.pathjoin(*entry[0][:2]))
+                if changed or self.include_unchanged:
+                    yield result
+            self.searched_exact_paths.add(path_utf8)
+
+    def _path_info(self, utf8_path, unicode_path):
+        """Generate path_info for unicode_path.
+
+        :return: None if unicode_path does not exist, or a path_info tuple.
+        """
+        abspath = self.tree.abspath(unicode_path)
+        try:
+            stat = os.lstat(abspath)
+        except OSError, e:
+            if e.errno == errno.ENOENT:
+                # the path does not exist.
+                return None
+            else:
+                raise
+        utf8_basename = utf8_path.rsplit('/', 1)[-1]
+        dir_info = (utf8_path, utf8_basename,
+            osutils.file_kind_from_stat_mode(stat.st_mode), stat,
+            abspath)
+        if dir_info[2] == 'directory':
+            if self.tree._directory_is_tree_reference(
+                unicode_path):
+                self.root_dir_info = self.root_dir_info[:2] + \
+                    ('tree-reference',) + self.root_dir_info[3:]
+        return dir_info
 
 
 # Try to load the compiled form if possible

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2009-08-14 09:31:37 +0000
+++ bzrlib/errors.py	2009-08-21 05:48:59 +0000
@@ -2920,8 +2920,9 @@
     _fmt = 'Cannot bind address "%(host)s:%(port)i": %(orig_error)s.'
 
     def __init__(self, host, port, orig_error):
+        # nb: in python2.4 socket.error doesn't have a useful repr
         BzrError.__init__(self, host=host, port=port,
-            orig_error=orig_error[1])
+            orig_error=repr(orig_error.args))
 
 
 class UnknownRules(BzrError):

=== modified file 'bzrlib/inventory.py'
--- bzrlib/inventory.py	2009-08-06 05:09:41 +0000
+++ bzrlib/inventory.py	2009-08-26 03:20:32 +0000
@@ -754,6 +754,8 @@
         >>> e = i.add(InventoryFile('foo-id', 'foo.c', parent_id='src-id'))
         >>> print i.id2path('foo-id')
         src/foo.c
+
+        :raises NoSuchId: If file_id is not present in the inventory.
         """
         # get all names, skipping root
         return '/'.join(reversed(

=== modified file 'bzrlib/merge.py'
--- bzrlib/merge.py	2009-07-02 13:07:14 +0000
+++ bzrlib/merge.py	2009-08-26 03:20:32 +0000
@@ -64,8 +64,12 @@
 
 
 def transform_tree(from_tree, to_tree, interesting_ids=None):
-    merge_inner(from_tree.branch, to_tree, from_tree, ignore_zero=True,
-                interesting_ids=interesting_ids, this_tree=from_tree)
+    from_tree.lock_tree_write()
+    try:
+        merge_inner(from_tree.branch, to_tree, from_tree, ignore_zero=True,
+                    interesting_ids=interesting_ids, this_tree=from_tree)
+    finally:
+        from_tree.unlock()
 
 
 class Merger(object):
@@ -102,6 +106,17 @@
         self._is_criss_cross = None
         self._lca_trees = None
 
+    def cache_trees_with_revision_ids(self, trees):
+        """Cache any tree in trees if it has a revision_id."""
+        for maybe_tree in trees:
+            if maybe_tree is None:
+                continue
+            try:
+                rev_id = maybe_tree.get_revision_id()
+            except AttributeError:
+                continue
+            self._cached_trees[rev_id] = maybe_tree
+
     @property
     def revision_graph(self):
         if self._revision_graph is None:
@@ -1516,6 +1531,7 @@
     get_revision_id = getattr(base_tree, 'get_revision_id', None)
     if get_revision_id is None:
         get_revision_id = base_tree.last_revision
+    merger.cache_trees_with_revision_ids([other_tree, base_tree, this_tree])
     merger.set_base_revision(get_revision_id(), this_branch)
     return merger.do_merge()
 

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2009-08-19 18:04:49 +0000
+++ bzrlib/repository.py	2009-08-26 03:20:32 +0000
@@ -807,6 +807,9 @@
                 seen_root = True
         self.new_inventory = None
         if len(inv_delta):
+            # This should perhaps be guarded by a check that the basis we
+            # commit against is the basis for the commit and if not do a delta
+            # against the basis.
             self._any_changes = True
         if not seen_root:
             # housekeeping root entry changes do not affect no-change commits.
@@ -3732,25 +3735,80 @@
             return False
         return True
 
-    def _get_delta_for_revision(self, tree, parent_ids, basis_id, cache):
+    def _get_trees(self, revision_ids, cache):
+        possible_trees = []
+        for rev_id in revision_ids:
+            if rev_id in cache:
+                possible_trees.append((rev_id, cache[rev_id]))
+            else:
+                # Not cached, but inventory might be present anyway.
+                try:
+                    tree = self.source.revision_tree(rev_id)
+                except errors.NoSuchRevision:
+                    # Nope, parent is ghost.
+                    pass
+                else:
+                    cache[rev_id] = tree
+                    possible_trees.append((rev_id, tree))
+        return possible_trees
+
+    def _get_delta_for_revision(self, tree, parent_ids, possible_trees):
         """Get the best delta and base for this revision.
 
         :return: (basis_id, delta)
         """
-        possible_trees = [(parent_id, cache[parent_id])
-                          for parent_id in parent_ids
-                           if parent_id in cache]
-        if len(possible_trees) == 0:
-            # There either aren't any parents, or the parents aren't in the
-            # cache, so just use the last converted tree
-            possible_trees.append((basis_id, cache[basis_id]))
         deltas = []
+        # Generate deltas against each tree, to find the shortest.
+        texts_possibly_new_in_tree = set()
         for basis_id, basis_tree in possible_trees:
             delta = tree.inventory._make_delta(basis_tree.inventory)
+            for old_path, new_path, file_id, new_entry in delta:
+                if new_path is None:
+                    # This file_id isn't present in the new rev, so we don't
+                    # care about it.
+                    continue
+                if not new_path:
+                    # Rich roots are handled elsewhere...
+                    continue
+                kind = new_entry.kind
+                if kind != 'directory' and kind != 'file':
+                    # No text record associated with this inventory entry.
+                    continue
+                # This is a directory or file that has changed somehow.
+                texts_possibly_new_in_tree.add((file_id, new_entry.revision))
             deltas.append((len(delta), basis_id, delta))
         deltas.sort()
         return deltas[0][1:]
 
+    def _fetch_parent_invs_for_stacking(self, parent_map, cache):
+        """Find all parent revisions that are absent, but for which the
+        inventory is present, and copy those inventories.
+
+        This is necessary to preserve correctness when the source is stacked
+        without fallbacks configured.  (Note that in cases like upgrade the
+        source may be not have _fallback_repositories even though it is
+        stacked.)
+        """
+        parent_revs = set()
+        for parents in parent_map.values():
+            parent_revs.update(parents)
+        present_parents = self.source.get_parent_map(parent_revs)
+        absent_parents = set(parent_revs).difference(present_parents)
+        parent_invs_keys_for_stacking = self.source.inventories.get_parent_map(
+            (rev_id,) for rev_id in absent_parents)
+        parent_inv_ids = [key[-1] for key in parent_invs_keys_for_stacking]
+        for parent_tree in self.source.revision_trees(parent_inv_ids):
+            current_revision_id = parent_tree.get_revision_id()
+            parents_parents_keys = parent_invs_keys_for_stacking[
+                (current_revision_id,)]
+            parents_parents = [key[-1] for key in parents_parents_keys]
+            basis_id = _mod_revision.NULL_REVISION
+            basis_tree = self.source.revision_tree(basis_id)
+            delta = parent_tree.inventory._make_delta(basis_tree.inventory)
+            self.target.add_inventory_by_delta(
+                basis_id, delta, current_revision_id, parents_parents)
+            cache[current_revision_id] = parent_tree
+
     def _fetch_batch(self, revision_ids, basis_id, cache):
         """Fetch across a few revisions.
 
@@ -3769,29 +3827,54 @@
         pending_deltas = []
         pending_revisions = []
         parent_map = self.source.get_parent_map(revision_ids)
+        self._fetch_parent_invs_for_stacking(parent_map, cache)
         for tree in self.source.revision_trees(revision_ids):
+            # Find a inventory delta for this revision.
+            # Find text entries that need to be copied, too.
             current_revision_id = tree.get_revision_id()
             parent_ids = parent_map.get(current_revision_id, ())
+            parent_trees = self._get_trees(parent_ids, cache)
+            possible_trees = list(parent_trees)
+            if len(possible_trees) == 0:
+                # There either aren't any parents, or the parents are ghosts,
+                # so just use the last converted tree.
+                possible_trees.append((basis_id, cache[basis_id]))
             basis_id, delta = self._get_delta_for_revision(tree, parent_ids,
-                                                           basis_id, cache)
+                                                           possible_trees)
             if self._converting_to_rich_root:
                 self._revision_id_to_root_id[current_revision_id] = \
                     tree.get_root_id()
-            # Find text entries that need to be copied
+            # Determine which texts are in present in this revision but not in
+            # any of the available parents.
+            texts_possibly_new_in_tree = set()
             for old_path, new_path, file_id, entry in delta:
-                if new_path is not None:
-                    if not new_path:
-                        # This is the root
-                        if not self.target.supports_rich_root():
-                            # The target doesn't support rich root, so we don't
-                            # copy
-                            continue
-                        if self._converting_to_rich_root:
-                            # This can't be copied normally, we have to insert
-                            # it specially
-                            root_keys_to_create.add((file_id, entry.revision))
-                            continue
-                    text_keys.add((file_id, entry.revision))
+                if new_path is None:
+                    # This file_id isn't present in the new rev
+                    continue
+                if not new_path:
+                    # This is the root
+                    if not self.target.supports_rich_root():
+                        # The target doesn't support rich root, so we don't
+                        # copy
+                        continue
+                    if self._converting_to_rich_root:
+                        # This can't be copied normally, we have to insert
+                        # it specially
+                        root_keys_to_create.add((file_id, entry.revision))
+                        continue
+                kind = entry.kind
+                texts_possibly_new_in_tree.add((file_id, entry.revision))
+            for basis_id, basis_tree in possible_trees:
+                basis_inv = basis_tree.inventory
+                for file_key in list(texts_possibly_new_in_tree):
+                    file_id, file_revision = file_key
+                    try:
+                        entry = basis_inv[file_id]
+                    except errors.NoSuchId:
+                        continue
+                    if entry.revision == file_revision:
+                        texts_possibly_new_in_tree.remove(file_key)
+            text_keys.update(texts_possibly_new_in_tree)
             revision = self.source.get_revision(current_revision_id)
             pending_deltas.append((basis_id, delta,
                 current_revision_id, revision.parent_ids))
@@ -3833,8 +3916,13 @@
             for parent_tree in self.source.revision_trees(parent_map):
                 current_revision_id = parent_tree.get_revision_id()
                 parents_parents = parent_map[current_revision_id]
+                possible_trees = self._get_trees(parents_parents, cache)
+                if len(possible_trees) == 0:
+                    # There either aren't any parents, or the parents are
+                    # ghosts, so just use the last converted tree.
+                    possible_trees.append((basis_id, cache[basis_id]))
                 basis_id, delta = self._get_delta_for_revision(parent_tree,
-                    parents_parents, basis_id, cache)
+                    parents_parents, possible_trees)
                 self.target.add_inventory_by_delta(
                     basis_id, delta, current_revision_id, parents_parents)
         # insert signatures and revisions

=== modified file 'bzrlib/revisiontree.py'
--- bzrlib/revisiontree.py	2009-07-06 20:48:00 +0000
+++ bzrlib/revisiontree.py	2009-08-26 03:20:32 +0000
@@ -242,28 +242,35 @@
         lookup_trees = [self.source]
         if extra_trees:
              lookup_trees.extend(extra_trees)
+        # The ids of items we need to examine to insure delta consistency.
+        precise_file_ids = set()
+        discarded_changes = {}
         if specific_files == []:
             specific_file_ids = []
         else:
             specific_file_ids = self.target.paths2ids(specific_files,
                 lookup_trees, require_versioned=require_versioned)
-
         # FIXME: It should be possible to delegate include_unchanged handling
         # to CHKInventory.iter_changes and do a better job there -- vila
         # 20090304
-        if include_unchanged:
-            changed_file_ids = []
+        changed_file_ids = set()
         for result in self.target.inventory.iter_changes(self.source.inventory):
-            if (specific_file_ids is not None
-                and not result[0] in specific_file_ids):
-                # CHKMap.iter_changes is clean and fast. Better filter out
-                # the specific files *after* it did its job.
-                continue
+            if specific_file_ids is not None:
+                file_id = result[0]
+                if file_id not in specific_file_ids:
+                    # A change from the whole tree that we don't want to show yet.
+                    # We may find that we need to show it for delta consistency, so
+                    # stash it.
+                    discarded_changes[result[0]] = result
+                    continue
+                new_parent_id = result[4][1]
+                precise_file_ids.add(new_parent_id)
             yield result
-            if include_unchanged:
-                # Keep track of yielded results (cheaper than building the
-                # whole inventory).
-                changed_file_ids.append(result[0])
+            changed_file_ids.add(result[0])
+        if specific_file_ids is not None:
+            for result in self._handle_precise_ids(precise_file_ids,
+                changed_file_ids, discarded_changes=discarded_changes):
+                yield result
         if include_unchanged:
             # CHKMap avoid being O(tree), so we go to O(tree) only if
             # required to.

=== modified file 'bzrlib/shelf.py'
--- bzrlib/shelf.py	2009-08-12 21:53:52 +0000
+++ bzrlib/shelf.py	2009-08-26 03:20:32 +0000
@@ -37,8 +37,10 @@
     def __init__(self, work_tree, target_tree, file_list=None):
         """Constructor.
 
-        :param work_tree: The working tree to apply changes to
+        :param work_tree: The working tree to apply changes to. This is not
+            required to be locked - a tree_write lock will be taken out.
         :param target_tree: The tree to make the working tree more similar to.
+            This is not required to be locked - a read_lock will be taken out.
         :param file_list: The files to make more similar to the target.
         """
         self.work_tree = work_tree
@@ -73,6 +75,12 @@
         """
         for (file_id, paths, changed, versioned, parents, names, kind,
              executable) in self.iter_changes:
+            # don't shelve add of tree root.  Working tree should never
+            # lack roots, and bzr misbehaves when they do.
+            # FIXME ADHB (2009-08-09): should still shelve adds of tree roots
+            # when a tree root was deleted / renamed.
+            if kind[0] is None and names[1] == '':
+                continue
             if kind[0] is None or versioned[0] == False:
                 self.creation[file_id] = (kind[1], names[1], parents[1],
                                           versioned)

=== modified file 'bzrlib/shelf_ui.py'
--- bzrlib/shelf_ui.py	2009-07-13 13:00:27 +0000
+++ bzrlib/shelf_ui.py	2009-08-26 03:20:32 +0000
@@ -149,6 +149,9 @@
                   message=None, directory='.', destroy=False):
         """Create a shelver from commandline arguments.
 
+        The returned shelver wil have a work_tree that is locked and should
+        be unlocked.
+
         :param revision: RevisionSpec of the revision to compare to.
         :param all: If True, shelve all changes without prompting.
         :param file_list: If supplied, only files in this list may be  shelved.
@@ -158,9 +161,16 @@
             changes.
         """
         tree, path = workingtree.WorkingTree.open_containing(directory)
-        target_tree = builtins._get_one_revision_tree('shelf2', revision,
-            tree.branch, tree)
-        files = builtins.safe_relpath_files(tree, file_list)
+        # Ensure that tree is locked for the lifetime of target_tree, as
+        # target tree may be reading from the same dirstate.
+        tree.lock_tree_write()
+        try:
+            target_tree = builtins._get_one_revision_tree('shelf2', revision,
+                tree.branch, tree)
+            files = builtins.safe_relpath_files(tree, file_list)
+        except:
+            tree.unlock()
+            raise
         return klass(tree, target_tree, diff_writer, all, all, files, message,
                      destroy)
 
@@ -313,32 +323,40 @@
     def from_args(klass, shelf_id=None, action='apply', directory='.'):
         """Create an unshelver from commandline arguments.
 
+        The returned shelver wil have a tree that is locked and should
+        be unlocked.
+
         :param shelf_id: Integer id of the shelf, as a string.
         :param action: action to perform.  May be 'apply', 'dry-run',
             'delete'.
         :param directory: The directory to unshelve changes into.
         """
         tree, path = workingtree.WorkingTree.open_containing(directory)
-        manager = tree.get_shelf_manager()
-        if shelf_id is not None:
-            try:
-                shelf_id = int(shelf_id)
-            except ValueError:
-                raise errors.InvalidShelfId(shelf_id)
-        else:
-            shelf_id = manager.last_shelf()
-            if shelf_id is None:
-                raise errors.BzrCommandError('No changes are shelved.')
-            trace.note('Unshelving changes with id "%d".' % shelf_id)
-        apply_changes = True
-        delete_shelf = True
-        read_shelf = True
-        if action == 'dry-run':
-            apply_changes = False
-            delete_shelf = False
-        if action == 'delete-only':
-            apply_changes = False
-            read_shelf = False
+        tree.lock_tree_write()
+        try:
+            manager = tree.get_shelf_manager()
+            if shelf_id is not None:
+                try:
+                    shelf_id = int(shelf_id)
+                except ValueError:
+                    raise errors.InvalidShelfId(shelf_id)
+            else:
+                shelf_id = manager.last_shelf()
+                if shelf_id is None:
+                    raise errors.BzrCommandError('No changes are shelved.')
+                trace.note('Unshelving changes with id "%d".' % shelf_id)
+            apply_changes = True
+            delete_shelf = True
+            read_shelf = True
+            if action == 'dry-run':
+                apply_changes = False
+                delete_shelf = False
+            if action == 'delete-only':
+                apply_changes = False
+                read_shelf = False
+        except:
+            tree.unlock()
+            raise
         return klass(tree, manager, shelf_id, apply_changes, delete_shelf,
                      read_shelf)
 
@@ -364,7 +382,7 @@
 
     def run(self):
         """Perform the unshelving operation."""
-        self.tree.lock_write()
+        self.tree.lock_tree_write()
         cleanups = [self.tree.unlock]
         try:
             if self.read_shelf:

=== modified file 'bzrlib/tests/blackbox/test_commit.py'
--- bzrlib/tests/blackbox/test_commit.py	2009-07-23 15:58:22 +0000
+++ bzrlib/tests/blackbox/test_commit.py	2009-08-26 03:20:32 +0000
@@ -273,13 +273,15 @@
         self.build_tree_contents([
             ('branch/foo.c', 'int main() {}'),
             ('branch/bar.c', 'int main() {}')])
-        inner_tree.add('foo.c')
-        inner_tree.add('bar.c')
+        inner_tree.add(['foo.c', 'bar.c'])
         # can't commit files in different trees; sane error
         self.run_bzr('commit -m newstuff branch/foo.c .', retcode=3)
+        # can commit to branch - records foo.c only
         self.run_bzr('commit -m newstuff branch/foo.c')
+        # can commit to branch - records bar.c
         self.run_bzr('commit -m newstuff branch')
-        self.run_bzr('commit -m newstuff branch', retcode=3)
+        # No changes left
+        self.run_bzr_error(["No changes to commit"], 'commit -m newstuff branch')
 
     def test_out_of_date_tree_commit(self):
         # check we get an error code and a clear message committing with an out

=== modified file 'bzrlib/tests/blackbox/test_non_ascii.py'
--- bzrlib/tests/blackbox/test_non_ascii.py	2009-07-31 17:53:02 +0000
+++ bzrlib/tests/blackbox/test_non_ascii.py	2009-08-20 04:09:58 +0000
@@ -287,7 +287,6 @@
         #       the files around as much.
         # Note that the tests don't actually fail, but if we don't set this
         # flag, we end up getting "Lock was not Unlocked" warnings
-        self.thisFailsStrictLockCheck() # clone?
 
         dirname = self.info['directory']
         self.run_bzr_decode(['push', dirname])

=== modified file 'bzrlib/tests/blackbox/test_push.py'
--- bzrlib/tests/blackbox/test_push.py	2009-07-31 17:53:02 +0000
+++ bzrlib/tests/blackbox/test_push.py	2009-08-20 04:09:58 +0000
@@ -134,7 +134,6 @@
 
     def test_push_without_tree(self):
         # bzr push from a branch that does not have a checkout should work.
-        self.thisFailsStrictLockCheck()
         b = self.make_branch('.')
         out, err = self.run_bzr('push pushed-location')
         self.assertEqual('', out)
@@ -146,7 +145,6 @@
         # bzr push of a branch with revisions to a new location
         # should print the number of revisions equal to the length of the
         # local branch.
-        self.thisFailsStrictLockCheck()
         t = self.make_branch_and_tree('tree')
         self.build_tree(['tree/file'])
         t.add('file')
@@ -157,7 +155,6 @@
 
     def test_push_only_pushes_history(self):
         # Knit branches should only push the history for the current revision.
-        self.thisFailsStrictLockCheck()
         format = bzrdir.BzrDirMetaFormat1()
         format.repository_format = knitrepo.RepositoryFormatKnit1()
         shared_repo = self.make_repository('repo', format=format, shared=True)
@@ -195,7 +192,6 @@
         self.assertTrue(pushed_repo.has_revision('b-1'))
 
     def test_push_funky_id(self):
-        self.thisFailsStrictLockCheck()
         t = self.make_branch_and_tree('tree')
         self.build_tree(['tree/filename'])
         t.add('filename', 'funky-chars<>%&;"\'')
@@ -203,7 +199,6 @@
         self.run_bzr('push -d tree new-tree')
 
     def test_push_dash_d(self):
-        self.thisFailsStrictLockCheck()
         t = self.make_branch_and_tree('from')
         t.commit(allow_pointless=True,
                 message='first commit')
@@ -292,7 +287,6 @@
 
     def test_push_create_prefix(self):
         """'bzr push --create-prefix' will create leading directories."""
-        self.thisFailsStrictLockCheck()
         tree = self.create_simple_tree()
 
         self.run_bzr_error(['Parent directory of ../new/tree does not exist'],
@@ -309,7 +303,6 @@
 
         By default, 'bzr push' will not use an existing, non-versioned dir.
         """
-        self.thisFailsStrictLockCheck()
         tree = self.create_simple_tree()
         self.build_tree(['target/'])
 
@@ -358,7 +351,6 @@
 
     def test_push_with_revisionspec(self):
         """We should be able to push a revision older than the tip."""
-        self.thisFailsStrictLockCheck()
         tree_from = self.make_branch_and_tree('from')
         tree_from.commit("One.", rev_id="from-1")
         tree_from.commit("Two.", rev_id="from-2")
@@ -400,7 +392,6 @@
 
     def test_push_new_branch_stacked_on(self):
         """Pushing a new branch with --stacked-on creates a stacked branch."""
-        self.thisFailsStrictLockCheck()
         trunk_tree, branch_tree = self.create_trunk_and_feature_branch()
         # we publish branch_tree with a reference to the mainline.
         out, err = self.run_bzr(['push', '--stacked-on', trunk_tree.branch.base,
@@ -413,7 +404,6 @@
 
     def test_push_new_branch_stacked_uses_parent_when_no_public_url(self):
         """When the parent has no public url the parent is used as-is."""
-        self.thisFailsStrictLockCheck()
         trunk_tree, branch_tree = self.create_trunk_and_feature_branch()
         # now we do a stacked push, which should determine the public location
         # for us.
@@ -427,7 +417,6 @@
 
     def test_push_new_branch_stacked_uses_parent_public(self):
         """Pushing a new branch with --stacked creates a stacked branch."""
-        self.thisFailsStrictLockCheck()
         trunk_tree, branch_tree = self.create_trunk_and_feature_branch()
         # the trunk is published on a web server
         self.transport_readonly_server = http_server.HttpServer
@@ -512,7 +501,6 @@
         out, err = self.run_bzr('push -d repo/local remote -r 3')
 
     def test_push_verbose_shows_log(self):
-        self.thisFailsStrictLockCheck()
         tree = self.make_branch_and_tree('source')
         tree.commit('rev1')
         out, err = self.run_bzr('push -v -d source target')
@@ -526,7 +514,6 @@
         self.assertNotContainsRe(out, 'rev1')
 
     def test_push_from_subdir(self):
-        self.thisFailsStrictLockCheck()
         t = self.make_branch_and_tree('tree')
         self.build_tree(['tree/dir/', 'tree/dir/file'])
         t.add('dir', 'dir/file')
@@ -670,24 +657,19 @@
         self.make_local_branch_and_tree()
 
     def test_push_default(self):
-        self.thisFailsStrictLockCheck()
         self.assertPushSucceeds([])
 
     def test_push_strict(self):
-        self.thisFailsStrictLockCheck()
         self.assertPushSucceeds(['--strict'])
 
     def test_push_no_strict(self):
-        self.thisFailsStrictLockCheck()
         self.assertPushSucceeds(['--no-strict'])
 
     def test_push_config_var_strict(self):
-        self.thisFailsStrictLockCheck()
         self.set_config_push_strict('true')
         self.assertPushSucceeds([])
 
     def test_push_config_var_no_strict(self):
-        self.thisFailsStrictLockCheck()
         self.set_config_push_strict('false')
         self.assertPushSucceeds([])
 
@@ -734,11 +716,9 @@
         self.assertPushFails([])
 
     def test_push_with_revision(self):
-        self.thisFailsStrictLockCheck()
         self.assertPushSucceeds(['-r', 'revid:added'], pushed_revid='added')
 
     def test_push_no_strict(self):
-        self.thisFailsStrictLockCheck()
         self.assertPushSucceeds(['--no-strict'])
 
     def test_push_strict_with_changes(self):
@@ -753,13 +733,11 @@
         self.assertPushFails([])
 
     def test_push_no_strict_command_line_override_config(self):
-        self.thisFailsStrictLockCheck()
         self.set_config_push_strict('yES')
         self.assertPushFails([])
         self.assertPushSucceeds(['--no-strict'])
 
     def test_push_strict_command_line_override_config(self):
-        self.thisFailsStrictLockCheck()
         self.set_config_push_strict('oFF')
         self.assertPushFails(['--strict'])
         self.assertPushSucceeds([])

=== modified file 'bzrlib/tests/blackbox/test_revert.py'
--- bzrlib/tests/blackbox/test_revert.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/blackbox/test_revert.py	2009-08-26 03:20:32 +0000
@@ -96,7 +96,9 @@
         self.failUnlessExists('a/b')
         self.failUnlessExists('a/d')
         self.failIfExists('a/g')
-        self.failUnlessExists('j')
+        self.expectFailure(
+            "j is in the delta revert applies because j was renamed too",
+            self.failUnlessExists, 'j')
         self.failUnlessExists('h')
         self.run_bzr('revert f')
         self.failIfExists('j')

=== modified file 'bzrlib/tests/blackbox/test_tags.py'
--- bzrlib/tests/blackbox/test_tags.py	2009-07-31 16:59:13 +0000
+++ bzrlib/tests/blackbox/test_tags.py	2009-08-20 04:09:58 +0000
@@ -74,7 +74,6 @@
         out, err = self.run_bzr('tag -d branch NEWTAG --force')
 
     def test_branch_push_pull_merge_copies_tags(self):
-        self.thisFailsStrictLockCheck()
         t = self.make_branch_and_tree('branch1')
         t.commit(allow_pointless=True, message='initial commit',
             rev_id='first-revid')

=== modified file 'bzrlib/tests/blackbox/test_too_much.py'
--- bzrlib/tests/blackbox/test_too_much.py	2009-07-31 16:59:13 +0000
+++ bzrlib/tests/blackbox/test_too_much.py	2009-08-20 04:09:58 +0000
@@ -280,7 +280,6 @@
 
     def test_push(self):
         # create a source branch
-        self.thisFailsStrictLockCheck()
         os.mkdir('my-branch')
         os.chdir('my-branch')
         self.example_branch()
@@ -613,7 +612,6 @@
         self.run_bzr(['check', url])
 
     def test_push(self):
-        self.thisFailsStrictLockCheck()
         # create a source branch
         os.mkdir('my-branch')
         os.chdir('my-branch')

=== modified file 'bzrlib/tests/blackbox/test_upgrade.py'
--- bzrlib/tests/blackbox/test_upgrade.py	2009-08-14 08:57:57 +0000
+++ bzrlib/tests/blackbox/test_upgrade.py	2009-08-21 02:10:06 +0000
@@ -184,7 +184,7 @@
         self.assertEqual('', err)
 
 
-class UpgradeRecommendedTests(TestCaseInTempDir):
+class UpgradeRecommendedTests(TestCaseWithTransport):
 
     def test_recommend_upgrade_wt4(self):
         # using a deprecated format gives a warning
@@ -200,3 +200,9 @@
         out, err = self.run_bzr('revno a')
         if err.find('upgrade') > -1:
             self.fail("message shouldn't suggest upgrade:\n%s" % err)
+
+    def test_upgrade_shared_repo(self):
+        repo = self.make_repository('repo', format='2a', shared=True)
+        branch = self.make_branch_and_tree('repo/branch', format="pack-0.92")
+        self.get_transport('repo/branch/.bzr/repository').delete_tree('.')
+        out, err = self.run_bzr(['upgrade'], working_dir='repo/branch')

=== modified file 'bzrlib/tests/blackbox/test_versioning.py'
--- bzrlib/tests/blackbox/test_versioning.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/blackbox/test_versioning.py	2009-08-26 03:20:32 +0000
@@ -22,10 +22,10 @@
 
 import os
 
-from bzrlib.tests import TestCaseInTempDir
 from bzrlib.branch import Branch
+from bzrlib.osutils import pathjoin
+from bzrlib.tests import TestCaseInTempDir, TestCaseWithTransport
 from bzrlib.trace import mutter
-from bzrlib.osutils import pathjoin
 from bzrlib.workingtree import WorkingTree
 
 
@@ -122,60 +122,51 @@
         check(b, False)
 
 
-class SubdirCommit(TestCaseInTempDir):
+class SubdirCommit(TestCaseWithTransport):
 
     def test_subdir_commit(self):
-        """Test committing a subdirectory, and committing within a directory."""
-        run_bzr = self.run_bzr
-        eq = self.assertEqual
-
+        """Test committing a subdirectory, and committing a directory."""
+        tree = self.make_branch_and_tree('.')
+        b = tree.branch
         self.build_tree(['a/', 'b/'])
-
-        run_bzr('init')
-        b = Branch.open(u'.')
-
-        for fn in ('a/one', 'b/two', 'top'):
-            file(fn, 'w').write('old contents')
-
-        run_bzr('add')
-        run_bzr(['commit', '-m', 'first revision'])
-
-        for fn in ('a/one', 'b/two', 'top'):
-            file(fn, 'w').write('new contents')
+        def set_contents(contents):
+            self.build_tree_contents([
+                ('a/one', contents),
+                ('b/two', contents),
+                ('top', contents),
+                ])
+        set_contents('old contents')
+        tree.smart_add(['.'])
+        tree.commit('first revision')
+        set_contents('new contents')
 
         mutter('start selective subdir commit')
-        run_bzr(['commit', 'a', '-m', 'commit a only'])
+        self.run_bzr(['commit', 'a', '-m', 'commit a only'])
 
-        old = b.repository.revision_tree(b.get_rev_id(1))
         new = b.repository.revision_tree(b.get_rev_id(2))
         new.lock_read()
 
-        eq(new.get_file_by_path('b/two').read(), 'old contents')
-        eq(new.get_file_by_path('top').read(), 'old contents')
-        eq(new.get_file_by_path('a/one').read(), 'new contents')
+        self.assertEqual(new.get_file_by_path('b/two').read(), 'old contents')
+        self.assertEqual(new.get_file_by_path('top').read(), 'old contents')
+        self.assertEqual(new.get_file_by_path('a/one').read(), 'new contents')
         new.unlock()
 
         os.chdir('a')
         # commit from here should do nothing
-        run_bzr(['commit', '.', '-m', 'commit subdir only', '--unchanged'])
+        self.run_bzr(['commit', '.', '-m', 'commit subdir only', '--unchanged'])
         v3 = b.repository.revision_tree(b.get_rev_id(3))
         v3.lock_read()
-        eq(v3.get_file_by_path('b/two').read(), 'old contents')
-        eq(v3.get_file_by_path('top').read(), 'old contents')
-        eq(v3.get_file_by_path('a/one').read(), 'new contents')
+        self.assertEqual(v3.get_file_by_path('b/two').read(), 'old contents')
+        self.assertEqual(v3.get_file_by_path('top').read(), 'old contents')
+        self.assertEqual(v3.get_file_by_path('a/one').read(), 'new contents')
         v3.unlock()
 
         # commit in subdirectory commits whole tree
-        run_bzr(['commit', '-m', 'commit whole tree from subdir'])
+        self.run_bzr(['commit', '-m', 'commit whole tree from subdir'])
         v4 = b.repository.revision_tree(b.get_rev_id(4))
         v4.lock_read()
-        eq(v4.get_file_by_path('b/two').read(), 'new contents')
-        eq(v4.get_file_by_path('top').read(), 'new contents')
+        self.assertEqual(v4.get_file_by_path('b/two').read(), 'new contents')
+        self.assertEqual(v4.get_file_by_path('top').read(), 'new contents')
         v4.unlock()
 
         # TODO: factor out some kind of assert_tree_state() method
-
-
-if __name__ == '__main__':
-    import unittest
-    unittest.main()

=== modified file 'bzrlib/tests/per_branch/test_branch.py'
--- bzrlib/tests/per_branch/test_branch.py	2009-07-31 16:59:13 +0000
+++ bzrlib/tests/per_branch/test_branch.py	2009-08-20 04:09:58 +0000
@@ -306,7 +306,6 @@
                          branch.repository.get_signature_text('A'))
 
     def test_branch_keeps_signatures(self):
-        self.thisFailsStrictLockCheck()
         wt = self.make_branch_and_tree('source')
         wt.commit('A', allow_pointless=True, rev_id='A')
         repo = wt.branch.repository

=== modified file 'bzrlib/tests/per_branch/test_create_clone.py'
--- bzrlib/tests/per_branch/test_create_clone.py	2009-07-31 16:59:13 +0000
+++ bzrlib/tests/per_branch/test_create_clone.py	2009-08-20 04:09:58 +0000
@@ -26,7 +26,6 @@
 class TestCreateClone(TestCaseWithBranch):
 
     def test_create_clone_on_transport_missing_parent_dir(self):
-        self.thisFailsStrictLockCheck()
         tree = self.make_branch_and_tree('source')
         tree.commit('a commit')
         source = tree.branch
@@ -36,7 +35,6 @@
         self.assertFalse(self.get_transport('.').has('subdir'))
 
     def test_create_clone_on_transport_missing_parent_dir_create(self):
-        self.thisFailsStrictLockCheck()
         tree = self.make_branch_and_tree('source')
         tree.commit('a commit')
         source = tree.branch
@@ -48,7 +46,6 @@
             result.bzrdir.root_transport.base)
 
     def test_create_clone_on_transport_use_existing_dir_false(self):
-        self.thisFailsStrictLockCheck()
         tree = self.make_branch_and_tree('source')
         tree.commit('a commit')
         source = tree.branch
@@ -59,7 +56,6 @@
         self.assertFalse(target_transport.has(".bzr"))
 
     def test_create_clone_on_transport_use_existing_dir_true(self):
-        self.thisFailsStrictLockCheck()
         tree = self.make_branch_and_tree('source')
         tree.commit('a commit')
         source = tree.branch
@@ -70,7 +66,6 @@
         self.assertEqual(source.last_revision(), result.last_revision())
 
     def test_create_clone_on_transport_no_revision_id(self):
-        self.thisFailsStrictLockCheck()
         tree = self.make_branch_and_tree('source')
         tree.commit('a commit')
         source = tree.branch
@@ -79,7 +74,6 @@
         self.assertEqual(source.last_revision(), result.last_revision())
 
     def test_create_clone_on_transport_revision_id(self):
-        self.thisFailsStrictLockCheck()
         tree = self.make_branch_and_tree('source')
         old_revid = tree.commit('a commit')
         source_tip = tree.commit('a second commit')
@@ -93,7 +87,6 @@
         self.assertFalse(result.repository.has_revision(source_tip))
 
     def test_create_clone_on_transport_stacked(self):
-        self.thisFailsStrictLockCheck()
         tree = self.make_branch_and_tree('source')
         tree.commit('a commit')
         trunk = tree.branch.create_clone_on_transport(
@@ -107,7 +100,6 @@
         self.assertEqual(trunk.base, result.get_stacked_on_url())
 
     def test_create_clone_of_multiple_roots(self):
-        self.thisFailsStrictLockCheck()
         try:
             builder = self.make_branch_builder('local')
         except (errors.TransportNotPossible, errors.UninitializableFormat):
@@ -129,7 +121,6 @@
         self.hook_calls.append(pre_change_params)
 
     def test_create_clone_on_transport_stacked_hooks_get_stacked_branch(self):
-        self.thisFailsStrictLockCheck()
         tree = self.make_branch_and_tree('source')
         tree.commit('a commit')
         trunk = tree.branch.create_clone_on_transport(

=== modified file 'bzrlib/tests/per_branch/test_stacking.py'
--- bzrlib/tests/per_branch/test_stacking.py	2009-08-17 06:17:21 +0000
+++ bzrlib/tests/per_branch/test_stacking.py	2009-08-20 04:09:58 +0000
@@ -226,7 +226,6 @@
         return stacked_bzrdir
 
     def test_clone_from_stacked_branch_preserve_stacking(self):
-        self.thisFailsStrictLockCheck()
         # We can clone from the bzrdir of a stacked branch. If
         # preserve_stacking is True, the cloned branch is stacked on the
         # same branch as the original.
@@ -243,7 +242,6 @@
             pass
 
     def test_clone_from_branch_stacked_on_relative_url_preserve_stacking(self):
-        self.thisFailsStrictLockCheck()
         # If a branch's stacked-on url is relative, we can still clone
         # from it with preserve_stacking True and get a branch stacked
         # on an appropriately adjusted relative url.
@@ -258,7 +256,6 @@
             cloned_bzrdir.open_branch().get_stacked_on_url())
 
     def test_clone_from_stacked_branch_no_preserve_stacking(self):
-        self.thisFailsStrictLockCheck()
         try:
             stacked_bzrdir = self.make_stacked_bzrdir()
         except unstackable_format_errors, e:
@@ -272,7 +269,6 @@
 
     def test_no_op_preserve_stacking(self):
         """With no stacking, preserve_stacking should be a no-op."""
-        self.thisFailsStrictLockCheck()
         branch = self.make_branch('source')
         cloned_bzrdir = branch.bzrdir.clone('cloned', preserve_stacking=True)
         self.assertRaises((errors.NotStacked, errors.UnstackableBranchFormat),
@@ -309,7 +305,6 @@
         """Obey policy where possible, ignore otherwise."""
         if isinstance(self.branch_format, branch.BzrBranchFormat4):
             raise TestNotApplicable('Branch format 4 does not autoupgrade.')
-        self.thisFailsStrictLockCheck()
         source = self.make_branch('source')
         stack_on = self.make_stacked_on_matching(source)
         parent_bzrdir = self.make_bzrdir('.', format='default')
@@ -467,7 +462,6 @@
         self.assertEqual(['../stack-on'], hook_calls)
 
     def test_stack_on_repository_branch(self):
-        self.thisFailsStrictLockCheck()
         # Stacking should work when the repo isn't co-located with the
         # stack-on branch.
         try:

=== modified file 'bzrlib/tests/per_bzrdir/test_bzrdir.py'
--- bzrlib/tests/per_bzrdir/test_bzrdir.py	2009-08-17 07:46:03 +0000
+++ bzrlib/tests/per_bzrdir/test_bzrdir.py	2009-08-20 04:09:58 +0000
@@ -528,7 +528,6 @@
         self.assertEqual('1', target.open_branch().last_revision())
 
     def test_clone_bzrdir_tree_branch_repo(self):
-        self.thisFailsStrictLockCheck()
         tree = self.make_branch_and_tree('source')
         self.build_tree(['source/foo'])
         tree.add('foo')
@@ -569,7 +568,6 @@
         self.assertEqual(target_repo._format, source_branch.repository._format)
 
     def test_revert_inventory(self):
-        self.thisFailsStrictLockCheck()
         tree = self.make_branch_and_tree('source')
         self.build_tree(['source/foo'])
         tree.add('foo')
@@ -624,7 +622,6 @@
                                      ])
 
     def test_clone_bzrdir_tree_revision(self):
-        self.thisFailsStrictLockCheck()
         # test for revision limiting, [smoke test, not corner case checks].
         # make a tree with a revision with a last-revision
         # and clone it with a revision limit.

=== modified file 'bzrlib/tests/per_interbranch/test_push.py'
--- bzrlib/tests/per_interbranch/test_push.py	2009-08-14 13:55:30 +0000
+++ bzrlib/tests/per_interbranch/test_push.py	2009-08-20 04:09:58 +0000
@@ -361,7 +361,6 @@
             self.hook_calls)
 
     def test_post_push_nonempty_history(self):
-        self.thisFailsStrictLockCheck()
         target = self.make_to_branch_and_tree('target')
         target.lock_write()
         target.add('')

=== modified file 'bzrlib/tests/per_interrepository/test_fetch.py'
--- bzrlib/tests/per_interrepository/test_fetch.py	2009-08-14 00:55:42 +0000
+++ bzrlib/tests/per_interrepository/test_fetch.py	2009-08-26 03:20:32 +0000
@@ -17,7 +17,6 @@
 
 import sys
 
-import bzrlib
 from bzrlib import (
     errors,
     inventory,
@@ -266,6 +265,91 @@
         self.assertEqual(expected_texts, unstacked_repo.texts.keys())
         self.assertCanStreamRevision(unstacked_repo, 'third')
 
+    def test_fetch_from_stacked_to_stacked_copies_parent_inventories(self):
+        """Fetch from a stacked branch copies inventories for parents of
+        revisions at the stacking boundary.
+
+        Specifically, fetch will copy the parent inventories from the
+        source for which the corresponding revisions are not present.  This
+        will happen even when the source repository has no fallbacks configured
+        (as is the case during upgrade).
+        """
+        if not self.repository_format.supports_external_lookups:
+            raise TestNotApplicable("Need stacking support in the source.")
+        if not self.repository_format_to.supports_external_lookups:
+            raise TestNotApplicable("Need stacking support in the target.")
+        builder = self.make_branch_builder('branch')
+        builder.start_series()
+        builder.build_snapshot('base', None, [
+            ('add', ('', 'root-id', 'directory', '')),
+            ('add', ('file', 'file-id', 'file', 'content\n'))])
+        builder.build_snapshot('left', ['base'], [
+            ('modify', ('file-id', 'left content\n'))])
+        builder.build_snapshot('right', ['base'], [
+            ('modify', ('file-id', 'right content\n'))])
+        builder.build_snapshot('merge', ['left', 'right'], [
+            ('modify', ('file-id', 'left and right content\n'))])
+        builder.finish_series()
+        branch = builder.get_branch()
+        repo = self.make_repository('old-trunk')
+        # Make a pair of equivalent trunk repos in the from and to formats.
+        old_trunk = repo.bzrdir.create_branch()
+        old_trunk.repository.fetch(branch.repository, 'left')
+        old_trunk.repository.fetch(branch.repository, 'right')
+        repo = self.make_to_repository('new-trunk')
+        new_trunk = repo.bzrdir.create_branch()
+        new_trunk.repository.fetch(branch.repository, 'left')
+        new_trunk.repository.fetch(branch.repository, 'right')
+        # Make the source; a repo stacked on old_trunk contained just the data
+        # for 'merge'.
+        repo = self.make_repository('old-stacked')
+        old_stacked_branch = repo.bzrdir.create_branch()
+        old_stacked_branch.set_stacked_on_url(old_trunk.base)
+        old_stacked_branch.repository.fetch(branch.repository, 'merge')
+        # Make the target, a repo stacked on new_trunk.
+        repo = self.make_to_repository('new-stacked')
+        new_stacked_branch = repo.bzrdir.create_branch()
+        new_stacked_branch.set_stacked_on_url(new_trunk.base)
+        old_unstacked_repo = old_stacked_branch.bzrdir.open_repository()
+        new_unstacked_repo = new_stacked_branch.bzrdir.open_repository()
+        # Reopen the source and target repos without any fallbacks, and fetch
+        # 'merge'.
+        new_unstacked_repo.fetch(old_unstacked_repo, 'merge')
+        # Now check the results.  new_unstacked_repo should contain all the
+        # data necessary to stream 'merge' (i.e. the parent inventories).
+        new_unstacked_repo.lock_read()
+        self.addCleanup(new_unstacked_repo.unlock)
+        self.assertFalse(new_unstacked_repo.has_revision('left'))
+        self.assertFalse(new_unstacked_repo.has_revision('right'))
+        self.assertEqual(
+            set([('left',), ('right',), ('merge',)]),
+            new_unstacked_repo.inventories.keys())
+        # And the basis inventories have been copied correctly
+        new_trunk.lock_read()
+        self.addCleanup(new_trunk.unlock)
+        left_tree, right_tree = new_trunk.repository.revision_trees(
+            ['left', 'right'])
+        new_stacked_branch.lock_read()
+        self.addCleanup(new_stacked_branch.unlock)
+        (stacked_left_tree,
+         stacked_right_tree) = new_stacked_branch.repository.revision_trees(
+            ['left', 'right'])
+        self.assertEqual(left_tree.inventory, stacked_left_tree.inventory)
+        self.assertEqual(right_tree.inventory, stacked_right_tree.inventory)
+        # Finally, it's not enough to see that the basis inventories are
+        # present.  The texts introduced in merge (and only those) should be
+        # present, and also generating a stream should succeed without blowing
+        # up.
+        self.assertTrue(new_unstacked_repo.has_revision('merge'))
+        expected_texts = set([('file-id', 'merge')])
+        if new_stacked_branch.repository.texts.get_parent_map([('root-id',
+            'merge')]):
+            # If a (root-id,merge) text exists, it should be in the stacked
+            # repo.
+            expected_texts.add(('root-id', 'merge'))
+        self.assertEqual(expected_texts, new_unstacked_repo.texts.keys())
+        self.assertCanStreamRevision(new_unstacked_repo, 'merge')
+
     def test_fetch_missing_basis_text(self):
         """If fetching a delta, we should die if a basis is not present."""
         tree = self.make_branch_and_tree('tree')

=== modified file 'bzrlib/tests/per_intertree/test_compare.py'
--- bzrlib/tests/per_intertree/test_compare.py	2009-07-17 06:04:35 +0000
+++ bzrlib/tests/per_intertree/test_compare.py	2009-08-26 03:20:32 +0000
@@ -37,8 +37,6 @@
 #        -> that is, when the renamed parent is not processed by the function.
 # TODO: test items are only emitted once when a specific_files list names a dir
 #       whose parent is now a child.
-# TODO: test specific_files when the target tree has a file and the source a
-#       dir with children, same id and same path.
 # TODO: test comparisons between trees with different root ids. mbp 20070301
 #
 # TODO: More comparisons between trees with subtrees in different states.
@@ -217,13 +215,29 @@
         d = self.intertree_class(tree1, tree2).compare(
             specific_files=['a', 'b/c'])
         self.assertEqual(
-            [('a', 'a-id', 'file'), ('b/c', 'c-id', 'file')],
+            [('a', 'a-id', 'file'),  (u'b', 'b-id', 'directory'),
+             ('b/c', 'c-id', 'file')],
             d.added)
         self.assertEqual([], d.modified)
         self.assertEqual([], d.removed)
         self.assertEqual([], d.renamed)
         self.assertEqual([], d.unchanged)
 
+    def test_empty_to_abc_content_c_only(self):
+        tree1 = self.make_branch_and_tree('1')
+        tree2 = self.make_to_branch_and_tree('2')
+        tree1 = self.get_tree_no_parents_no_content(tree1)
+        tree2 = self.get_tree_no_parents_abc_content(tree2)
+        tree1, tree2 = self.mutable_trees_to_test_trees(self, tree1, tree2)
+        d = self.intertree_class(tree1, tree2).compare(
+            specific_files=['b/c'])
+        self.assertEqual(
+            [(u'b', 'b-id', 'directory'), ('b/c', 'c-id', 'file')], d.added)
+        self.assertEqual([], d.modified)
+        self.assertEqual([], d.removed)
+        self.assertEqual([], d.renamed)
+        self.assertEqual([], d.unchanged)
+
     def test_empty_to_abc_content_b_only(self):
         """Restricting to a dir matches the children of the dir."""
         tree1 = self.make_branch_and_tree('1')
@@ -233,7 +247,7 @@
         tree1, tree2 = self.mutable_trees_to_test_trees(self, tree1, tree2)
         d = self.intertree_class(tree1, tree2).compare(specific_files=['b'])
         self.assertEqual(
-            [('b', 'b-id', 'directory'),('b/c', 'c-id', 'file')],
+            [('b', 'b-id', 'directory'), ('b/c', 'c-id', 'file')],
             d.added)
         self.assertEqual([], d.modified)
         self.assertEqual([], d.removed)
@@ -350,6 +364,38 @@
 class TestIterChanges(TestCaseWithTwoTrees):
     """Test the comparison iterator"""
 
+    def assertEqualIterChanges(self, left_changes, right_changes):
+        """Assert that left_changes == right_changes.
+
+        :param left_changes: A list of the output from iter_changes.
+        :param right_changes: A list of the output from iter_changes.
+        """
+        left_changes = sorted(left_changes)
+        right_changes = sorted(right_changes)
+        if left_changes == right_changes:
+            return
+        # setify to get item by item differences, but we can only do this
+        # when all the ids are unique on both sides.
+        left_dict = dict((item[0], item) for item in left_changes)
+        right_dict = dict((item[0], item) for item in right_changes)
+        if (len(left_dict) != len(left_changes) or
+            len(right_dict) != len(right_changes)):
+            # Can't do a direct comparison. We could do a sequence diff, but
+            # for now just do a regular assertEqual for now.
+            self.assertEqual(left_changes, right_changes)
+        keys = set(left_dict).union(set(right_dict))
+        different = []
+        same = []
+        for key in keys:
+            left_item = left_dict.get(key)
+            right_item = right_dict.get(key)
+            if left_item == right_item:
+                same.append(str(left_item))
+            else:
+                different.append(" %s\n %s" % (left_item, right_item))
+        self.fail("iter_changes output different. Unchanged items:\n" +
+            "\n".join(same) + "\nChanged items:\n" + "\n".join(different))
+
     def do_iter_changes(self, tree1, tree2, **extra_args):
         """Helper to run iter_changes from tree1 to tree2.
 
@@ -379,12 +425,14 @@
                 # Neither tree can be used
                 return
         tree1.lock_read()
-        tree2.lock_read()
         try:
-            return tree2.has_changes(tree1)
+            tree2.lock_read()
+            try:
+                return tree2.has_changes(tree1)
+            finally:
+                tree2.unlock()
         finally:
             tree1.unlock()
-            tree2.unlock()
 
     def mutable_trees_to_locked_test_trees(self, tree1, tree2):
         """Convert the working trees into test trees.
@@ -572,10 +620,13 @@
         tree2 = self.get_tree_no_parents_abc_content(tree2)
         tree1, tree2 = self.mutable_trees_to_locked_test_trees(tree1, tree2)
         self.assertEqual(
-            [self.added(tree2, 'a-id')],
+            sorted([self.added(tree2, 'root-id'),
+             self.added(tree2, 'a-id'),
+             self.deleted(tree1, 'empty-root-id')]),
             self.do_iter_changes(tree1, tree2, specific_files=['a']))
 
-    def test_abc_content_to_empty_to_abc_content_a_only(self):
+    def test_abc_content_to_empty_a_only(self):
+        # For deletes we don't need to pickup parents.
         tree1 = self.make_branch_and_tree('1')
         tree2 = self.make_to_branch_and_tree('2')
         tree1 = self.get_tree_no_parents_abc_content(tree1)
@@ -585,13 +636,26 @@
             [self.deleted(tree1, 'a-id')],
             self.do_iter_changes(tree1, tree2, specific_files=['a']))
 
+    def test_abc_content_to_empty_b_only(self):
+        # When b stops being a directory we have to pick up b/c as well.
+        tree1 = self.make_branch_and_tree('1')
+        tree2 = self.make_to_branch_and_tree('2')
+        tree1 = self.get_tree_no_parents_abc_content(tree1)
+        tree2 = self.get_tree_no_parents_no_content(tree2)
+        tree1, tree2 = self.mutable_trees_to_locked_test_trees(tree1, tree2)
+        self.assertEqual(
+            [self.deleted(tree1, 'b-id'), self.deleted(tree1, 'c-id')],
+            self.do_iter_changes(tree1, tree2, specific_files=['b']))
+
     def test_empty_to_abc_content_a_and_c_only(self):
         tree1 = self.make_branch_and_tree('1')
         tree2 = self.make_to_branch_and_tree('2')
         tree1 = self.get_tree_no_parents_no_content(tree1)
         tree2 = self.get_tree_no_parents_abc_content(tree2)
         tree1, tree2 = self.mutable_trees_to_locked_test_trees(tree1, tree2)
-        expected_result = [self.added(tree2, 'a-id'), self.added(tree2, 'c-id')]
+        expected_result = sorted([self.added(tree2, 'root-id'),
+            self.added(tree2, 'a-id'), self.added(tree2, 'b-id'),
+            self.added(tree2, 'c-id'), self.deleted(tree1, 'empty-root-id')])
         self.assertEqual(expected_result,
             self.do_iter_changes(tree1, tree2, specific_files=['a', 'b/c']))
 
@@ -601,17 +665,10 @@
         tree1 = self.get_tree_no_parents_abc_content(tree1)
         tree2 = self.get_tree_no_parents_no_content(tree2)
         tree1, tree2 = self.mutable_trees_to_locked_test_trees(tree1, tree2)
-        def deleted(file_id):
-            entry = tree1.inventory[file_id]
-            path = tree1.id2path(file_id)
-            return (file_id, (path, None), True, (True, False),
-                    (entry.parent_id, None),
-                    (entry.name, None), (entry.kind, None),
-                    (entry.executable, None))
         expected_results = sorted([
             self.added(tree2, 'empty-root-id'),
-            deleted('root-id'), deleted('a-id'),
-            deleted('b-id'), deleted('c-id')])
+            self.deleted(tree1, 'root-id'), self.deleted(tree1, 'a-id'),
+            self.deleted(tree1, 'b-id'), self.deleted(tree1, 'c-id')])
         self.assertEqual(
             expected_results,
             self.do_iter_changes(tree1, tree2))
@@ -679,6 +736,206 @@
                            (False, False))],
                          self.do_iter_changes(tree1, tree2))
 
+    def test_specific_content_modification_grabs_parents(self):
+        # WHen the only direct change to a specified file is a content change,
+        # and its in a reparented subtree, the parents are grabbed.
+        tree1 = self.make_branch_and_tree('1')
+        tree1.mkdir('changing', 'parent-id')
+        tree1.mkdir('changing/unchanging', 'mid-id')
+        tree1.add(['changing/unchanging/file'], ['file-id'], ['file'])
+        tree1.put_file_bytes_non_atomic('file-id', 'a file')
+        tree2 = self.make_to_branch_and_tree('2')
+        tree2.set_root_id(tree1.get_root_id())
+        tree2.mkdir('changed', 'parent-id')
+        tree2.mkdir('changed/unchanging', 'mid-id')
+        tree2.add(['changed/unchanging/file'], ['file-id'], ['file'])
+        tree2.put_file_bytes_non_atomic('file-id', 'changed content')
+        tree1, tree2 = self.mutable_trees_to_test_trees(self, tree1, tree2)
+        # parent-id has changed, as has file-id
+        root_id = tree1.path2id('')
+        self.assertEqualIterChanges(
+            [self.renamed(tree1, tree2, 'parent-id', False),
+             self.renamed(tree1, tree2, 'file-id', True)],
+             self.do_iter_changes(tree1, tree2,
+             specific_files=['changed/unchanging/file']))
+
+    def test_specific_content_modification_grabs_parents_root_changes(self):
+        # WHen the only direct change to a specified file is a content change,
+        # and its in a reparented subtree, the parents are grabbed, even if
+        # that includes the root.
+        tree1 = self.make_branch_and_tree('1')
+        tree1.set_root_id('old')
+        tree1.mkdir('changed', 'parent-id')
+        tree1.mkdir('changed/unchanging', 'mid-id')
+        tree1.add(['changed/unchanging/file'], ['file-id'], ['file'])
+        tree1.put_file_bytes_non_atomic('file-id', 'a file')
+        tree2 = self.make_to_branch_and_tree('2')
+        tree2.set_root_id('new')
+        tree2.mkdir('changed', 'parent-id')
+        tree2.mkdir('changed/unchanging', 'mid-id')
+        tree2.add(['changed/unchanging/file'], ['file-id'], ['file'])
+        tree2.put_file_bytes_non_atomic('file-id', 'changed content')
+        tree1, tree2 = self.mutable_trees_to_locked_test_trees(tree1, tree2)
+        # old is gone, new is added, parent-id has changed(reparented), as has
+        # file-id(content)
+        root_id = tree1.path2id('')
+        self.assertEqualIterChanges(
+            [self.renamed(tree1, tree2, 'parent-id', False),
+             self.added(tree2, 'new'),
+             self.deleted(tree1, 'old'),
+             self.renamed(tree1, tree2, 'file-id', True)],
+             self.do_iter_changes(tree1, tree2,
+             specific_files=['changed/unchanging/file']))
+
+    def test_specific_with_rename_under_new_dir_reports_new_dir(self):
+        tree1 = self.make_branch_and_tree('1')
+        tree2 = self.make_to_branch_and_tree('2')
+        tree1 = self.get_tree_no_parents_abc_content(tree1)
+        tree2 = self.get_tree_no_parents_abc_content_7(tree2)
+        tree1, tree2 = self.mutable_trees_to_test_trees(self, tree1, tree2)
+        # d(d-id) is new, e is b-id renamed. 
+        root_id = tree1.path2id('')
+        self.assertEqualIterChanges(
+            [self.renamed(tree1, tree2, 'b-id', False),
+             self.added(tree2, 'd-id')],
+             self.do_iter_changes(tree1, tree2, specific_files=['d/e']))
+
+    def test_specific_with_rename_under_dir_under_new_dir_reports_new_dir(self):
+        tree1 = self.make_branch_and_tree('1')
+        tree2 = self.make_to_branch_and_tree('2')
+        tree1 = self.get_tree_no_parents_abc_content(tree1)
+        tree2 = self.get_tree_no_parents_abc_content_7(tree2)
+        tree2.rename_one('a', 'd/e/a')
+        tree1, tree2 = self.mutable_trees_to_test_trees(self, tree1, tree2)
+        # d is new, d/e is b-id renamed, d/e/a is a-id renamed 
+        root_id = tree1.path2id('')
+        self.assertEqualIterChanges(
+            [self.renamed(tree1, tree2, 'b-id', False),
+             self.added(tree2, 'd-id'),
+             self.renamed(tree1, tree2, 'a-id', False)],
+             self.do_iter_changes(tree1, tree2, specific_files=['d/e/a']))
+
+    def test_specific_old_parent_same_path_new_parent(self):
+        # when a parent is new at its path, if the path was used in the source
+        # it must be emitted as a change.
+        tree1 = self.make_branch_and_tree('1')
+        tree1.add(['a'], ['a-id'], ['file'])
+        tree1.put_file_bytes_non_atomic('a-id', 'a file')
+        tree2 = self.make_to_branch_and_tree('2')
+        tree2.set_root_id(tree1.get_root_id())
+        tree2.mkdir('a', 'b-id')
+        tree2.add(['a/c'], ['c-id'], ['file'])
+        tree2.put_file_bytes_non_atomic('c-id', 'another file')
+        tree1, tree2 = self.mutable_trees_to_locked_test_trees(tree1, tree2)
+        # a-id is gone, b-id and c-id are added.
+        self.assertEqualIterChanges(
+            [self.deleted(tree1, 'a-id'),
+             self.added(tree2, 'b-id'),
+             self.added(tree2, 'c-id')],
+             self.do_iter_changes(tree1, tree2, specific_files=['a/c']))
+
+    def test_specific_old_parent_becomes_file(self):
+        # When an old parent included because of a path conflict becomes a
+        # non-directory, its children have to be all included in the delta.
+        tree1 = self.make_branch_and_tree('1')
+        tree1.mkdir('a', 'a-old-id')
+        tree1.mkdir('a/reparented', 'reparented-id')
+        tree1.mkdir('a/deleted', 'deleted-id')
+        tree2 = self.make_to_branch_and_tree('2')
+        tree2.set_root_id(tree1.get_root_id())
+        tree2.mkdir('a', 'a-new-id')
+        tree2.mkdir('a/reparented', 'reparented-id')
+        tree2.add(['b'], ['a-old-id'], ['file'])
+        tree2.put_file_bytes_non_atomic('a-old-id', '')
+        tree1, tree2 = self.mutable_trees_to_locked_test_trees(tree1, tree2)
+        # a-old-id is kind-changed, a-new-id is added, reparented-id is renamed,
+        # deleted-id is gone
+        self.assertEqualIterChanges(
+            [self.kind_changed(tree1, tree2, 'a-old-id'),
+             self.added(tree2, 'a-new-id'),
+             self.renamed(tree1, tree2, 'reparented-id', False),
+             self.deleted(tree1, 'deleted-id')],
+             self.do_iter_changes(tree1, tree2,
+                specific_files=['a/reparented']))
+
+    def test_specific_old_parent_is_deleted(self):
+        # When an old parent included because of a path conflict is removed,
+        # its children have to be all included in the delta.
+        tree1 = self.make_branch_and_tree('1')
+        tree1.mkdir('a', 'a-old-id')
+        tree1.mkdir('a/reparented', 'reparented-id')
+        tree1.mkdir('a/deleted', 'deleted-id')
+        tree2 = self.make_to_branch_and_tree('2')
+        tree2.set_root_id(tree1.get_root_id())
+        tree2.mkdir('a', 'a-new-id')
+        tree2.mkdir('a/reparented', 'reparented-id')
+        tree1, tree2 = self.mutable_trees_to_locked_test_trees(tree1, tree2)
+        # a-old-id is gone, a-new-id is added, reparented-id is renamed,
+        # deleted-id is gone
+        self.assertEqualIterChanges(
+            [self.deleted(tree1, 'a-old-id'),
+             self.added(tree2, 'a-new-id'),
+             self.renamed(tree1, tree2, 'reparented-id', False),
+             self.deleted(tree1, 'deleted-id')],
+             self.do_iter_changes(tree1, tree2,
+                specific_files=['a/reparented']))
+
+    def test_specific_old_parent_child_collides_with_unselected_new(self):
+        # When the child of an old parent because of a path conflict becomes a
+        # path conflict with some unselected item in the source, that item also
+        # needs to be included (because otherwise the output of applying the
+        # delta to the source would have two items at that path).
+        tree1 = self.make_branch_and_tree('1')
+        tree1.mkdir('a', 'a-old-id')
+        tree1.mkdir('a/reparented', 'reparented-id')
+        tree1.mkdir('collides', 'collides-id')
+        tree2 = self.make_to_branch_and_tree('2')
+        tree2.set_root_id(tree1.get_root_id())
+        tree2.mkdir('a', 'a-new-id')
+        tree2.mkdir('a/selected', 'selected-id')
+        tree2.mkdir('collides', 'reparented-id')
+        tree1, tree2 = self.mutable_trees_to_locked_test_trees(tree1, tree2)
+        # a-old-id is one, a-new-id is added, reparented-id is renamed,
+        # collides-id is gone, selected-id is new.
+        self.assertEqualIterChanges(
+            [self.deleted(tree1, 'a-old-id'),
+             self.added(tree2, 'a-new-id'),
+             self.renamed(tree1, tree2, 'reparented-id', False),
+             self.deleted(tree1, 'collides-id'),
+             self.added(tree2, 'selected-id')],
+             self.do_iter_changes(tree1, tree2,
+                specific_files=['a/selected']))
+
+    def test_specific_old_parent_child_dir_stops_being_dir(self):
+        # When the child of an old parent also stops being a directory, its
+        # children must also be included. This test checks that downward
+        # recursion is done appropriately by starting at a child of the root of
+        # a deleted subtree (a/reparented), and checking that a sibling
+        # directory (a/deleted) has its children included in the delta.
+        tree1 = self.make_branch_and_tree('1')
+        tree1.mkdir('a', 'a-old-id')
+        tree1.mkdir('a/reparented', 'reparented-id-1')
+        tree1.mkdir('a/deleted', 'deleted-id-1')
+        tree1.mkdir('a/deleted/reparented', 'reparented-id-2')
+        tree1.mkdir('a/deleted/deleted', 'deleted-id-2')
+        tree2 = self.make_to_branch_and_tree('2')
+        tree2.set_root_id(tree1.get_root_id())
+        tree2.mkdir('a', 'a-new-id')
+        tree2.mkdir('a/reparented', 'reparented-id-1')
+        tree2.mkdir('reparented', 'reparented-id-2')
+        tree1, tree2 = self.mutable_trees_to_locked_test_trees(tree1, tree2)
+        # a-old-id is gone, a-new-id is added, reparented-id-1, -2 are renamed,
+        # deleted-id-1 and -2 are gone.
+        self.assertEqualIterChanges(
+            [self.deleted(tree1, 'a-old-id'),
+             self.added(tree2, 'a-new-id'),
+             self.renamed(tree1, tree2, 'reparented-id-1', False),
+             self.renamed(tree1, tree2, 'reparented-id-2', False),
+             self.deleted(tree1, 'deleted-id-1'),
+             self.deleted(tree1, 'deleted-id-2')],
+             self.do_iter_changes(tree1, tree2,
+                specific_files=['a/reparented']))
+
     def test_file_rename_and_meta_modification(self):
         tree1 = self.make_branch_and_tree('1')
         tree2 = self.make_to_branch_and_tree('2')
@@ -1148,15 +1405,16 @@
 
         tree1, tree2 = self.mutable_trees_to_locked_test_trees(tree1, tree2)
         # We should notice that 'b' and all its children are deleted
-        expected = sorted([
+        expected = [
             self.content_changed(tree2, 'a-id'),
             self.content_changed(tree2, 'g-id'),
             self.deleted(tree1, 'b-id'),
             self.deleted(tree1, 'c-id'),
             self.deleted(tree1, 'd-id'),
             self.deleted(tree1, 'e-id'),
-            ])
-        self.assertEqual(expected, self.do_iter_changes(tree1, tree2))
+            ]
+        self.assertEqualIterChanges(expected,
+            self.do_iter_changes(tree1, tree2))
         self.check_has_changes(True, tree1, tree2)
 
     def test_added_unicode(self):
@@ -1273,9 +1531,9 @@
 
         self.assertEqual([self.renamed(tree1, tree2, rename_id, False)],
                          self.do_iter_changes(tree1, tree2))
-        self.assertEqual([self.renamed(tree1, tree2, rename_id, False)],
-                         self.do_iter_changes(tree1, tree2,
-                                              specific_files=[u'\u03b1']))
+        self.assertEqualIterChanges(
+            [self.renamed(tree1, tree2, rename_id, False)],
+            self.do_iter_changes(tree1, tree2, specific_files=[u'\u03b1']))
         self.check_has_changes(True, tree1, tree2)
 
     def test_unchanged_unicode(self):
@@ -1323,9 +1581,8 @@
             self.unchanged(tree1, subfile_id),
             ])
         self.assertEqual(expected,
-                         self.do_iter_changes(tree1, tree2,
-                                              specific_files=[u'\u03b1'],
-                                              include_unchanged=True))
+            self.do_iter_changes(tree1, tree2, specific_files=[u'\u03b1'],
+                include_unchanged=True))
 
     def test_unknown_unicode(self):
         tree1 = self.make_branch_and_tree('tree1')

=== modified file 'bzrlib/tests/per_repository_reference/test_get_rev_id_for_revno.py'
--- bzrlib/tests/per_repository_reference/test_get_rev_id_for_revno.py	2009-07-31 17:42:29 +0000
+++ bzrlib/tests/per_repository_reference/test_get_rev_id_for_revno.py	2009-08-20 04:09:58 +0000
@@ -24,7 +24,6 @@
 class TestGetRevIdForRevno(TestCaseWithExternalReferenceRepository):
 
     def test_uses_fallback(self):
-        self.thisFailsStrictLockCheck()
         tree = self.make_branch_and_tree('base')
         base = tree.branch.repository
         revid = tree.commit('one')

=== modified file 'bzrlib/tests/per_tree/__init__.py'
--- bzrlib/tests/per_tree/__init__.py	2009-07-10 10:46:00 +0000
+++ bzrlib/tests/per_tree/__init__.py	2009-08-26 03:20:32 +0000
@@ -201,6 +201,21 @@
         tt.apply()
         return self._convert_tree(tree, converter)
 
+    def get_tree_no_parents_abc_content_7(self, tree, converter=None):
+        """return a test tree with a, b/, d/e contents.
+
+        This variation adds a dir 'd' ('d-id'), renames b to d/e.
+        """
+        self._make_abc_tree(tree)
+        self.build_tree(['d/'], transport=tree.bzrdir.root_transport)
+        tree.add(['d'], ['d-id'])
+        tt = transform.TreeTransform(tree)
+        trans_id = tt.trans_id_tree_path('b')
+        parent_trans_id = tt.trans_id_tree_path('d')
+        tt.adjust_path('e', parent_trans_id, trans_id)
+        tt.apply()
+        return self._convert_tree(tree, converter)
+
     def get_tree_with_subdirs_and_all_content_types(self):
         """Return a test tree with subdirs and all content types.
         See get_tree_with_subdirs_and_all_supported_content_types for details.

=== modified file 'bzrlib/tests/per_workingtree/test_executable.py'
--- bzrlib/tests/per_workingtree/test_executable.py	2009-07-31 17:42:29 +0000
+++ bzrlib/tests/per_workingtree/test_executable.py	2009-08-20 04:09:58 +0000
@@ -30,8 +30,6 @@
 
     def setUp(self):
         super(TestExecutable, self).setUp()
-        self.thisFailsStrictLockCheck()
-
         self.a_id = "a-20051208024829-849e76f7968d7a86"
         self.b_id = "b-20051208024829-849e76f7968d7a86"
         wt = self.make_branch_and_tree('b1')

=== modified file 'bzrlib/tests/per_workingtree/test_set_root_id.py'
--- bzrlib/tests/per_workingtree/test_set_root_id.py	2009-07-31 17:42:29 +0000
+++ bzrlib/tests/per_workingtree/test_set_root_id.py	2009-08-21 01:48:13 +0000
@@ -23,8 +23,8 @@
 class TestSetRootId(TestCaseWithWorkingTree):
 
     def test_set_and_read_unicode(self):
-        # This write locks the local tree, and then grabs a read lock on a
-        # copy, which is bogus and the test just needs to be rewritten.
+        # This test tests that setting the root doesn't flush, so it
+        # deliberately tests concurrent access that isn't possible on windows.
         self.thisFailsStrictLockCheck()
         tree = self.make_branch_and_tree('a-tree')
         # setting the root id allows it to be read via get_root_id.

=== modified file 'bzrlib/tests/per_workingtree/test_workingtree.py'
--- bzrlib/tests/per_workingtree/test_workingtree.py	2009-07-31 17:42:29 +0000
+++ bzrlib/tests/per_workingtree/test_workingtree.py	2009-08-20 04:09:58 +0000
@@ -362,7 +362,6 @@
                             wt.get_parent_ids())
 
     def test_clone_preserves_content(self):
-        self.thisFailsStrictLockCheck()
         wt = self.make_branch_and_tree('source')
         self.build_tree(['added', 'deleted', 'notadded'],
                         transport=wt.bzrdir.transport.clone('..'))

=== modified file 'bzrlib/tests/test_errors.py'
--- bzrlib/tests/test_errors.py	2009-07-07 03:40:29 +0000
+++ bzrlib/tests/test_errors.py	2009-08-21 02:37:18 +0000
@@ -1,6 +1,4 @@
-# Copyright (C) 2006, 2007, 2008 Canonical Ltd
-#   Authors: Robert Collins <robert.collins@canonical.com>
-#            and others
+# Copyright (C) 2006, 2007, 2008, 2009 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,7 +16,9 @@
 
 """Tests for the formatting and construction of errors."""
 
+import socket
 import sys
+
 from bzrlib import (
     bzrdir,
     errors,
@@ -669,3 +669,10 @@
         e = ErrorWithBadFormat(not_thing='x')
         self.assertStartsWith(
             str(e), 'Unprintable exception ErrorWithBadFormat')
+
+    def test_cannot_bind_address(self):
+        # see <https://bugs.edge.launchpad.net/bzr/+bug/286871>
+        e = errors.CannotBindAddress('example.com', 22,
+            socket.error(13, 'Permission denied'))
+        self.assertContainsRe(str(e),
+            r'Cannot bind address "example\.com:22":.*Permission denied')

=== modified file 'bzrlib/tests/test_merge.py'
--- bzrlib/tests/test_merge.py	2009-07-31 17:42:29 +0000
+++ bzrlib/tests/test_merge.py	2009-08-26 03:20:32 +0000
@@ -117,7 +117,6 @@
 
     def test_create_rename(self):
         """Rename an inventory entry while creating the file"""
-        self.thisFailsStrictLockCheck()
         tree =self.make_branch_and_tree('.')
         file('name1', 'wb').write('Hello')
         tree.add('name1')
@@ -128,7 +127,6 @@
 
     def test_layered_rename(self):
         """Rename both child and parent at same time"""
-        self.thisFailsStrictLockCheck()
         tree =self.make_branch_and_tree('.')
         os.mkdir('dirname1')
         tree.add('dirname1')
@@ -215,19 +213,20 @@
         self.assertFileEqual('text2', 'tree/sub-tree/file')
 
     def test_merge_with_missing(self):
-        self.thisFailsStrictLockCheck()
         tree_a = self.make_branch_and_tree('tree_a')
         self.build_tree_contents([('tree_a/file', 'content_1')])
         tree_a.add('file')
         tree_a.commit('commit base')
         # basis_tree() is only guaranteed to be valid as long as it is actually
-        # the basis tree. This mutates the tree after grabbing basis, so go to
-        # the repository.
+        # the basis tree. This test commits to the tree after grabbing basis,
+        # so we go to the repository.
         base_tree = tree_a.branch.repository.revision_tree(tree_a.last_revision())
         tree_b = tree_a.bzrdir.sprout('tree_b').open_workingtree()
         self.build_tree_contents([('tree_a/file', 'content_2')])
         tree_a.commit('commit other')
         other_tree = tree_a.basis_tree()
+        # 'file' is now missing but isn't altered in any commit in b so no
+        # change should be applied.
         os.unlink('tree_b/file')
         merge_inner(tree_b.branch, other_tree, base_tree, this_tree=tree_b)
 
@@ -1235,6 +1234,27 @@
 
 class TestMergerInMemory(TestMergerBase):
 
+    def test_cache_trees_with_revision_ids_None(self):
+        merger = self.make_Merger(self.setup_simple_graph(), 'C-id')
+        original_cache = dict(merger._cached_trees)
+        merger.cache_trees_with_revision_ids([None])
+        self.assertEqual(original_cache, merger._cached_trees)
+
+    def test_cache_trees_with_revision_ids_no_revision_id(self):
+        merger = self.make_Merger(self.setup_simple_graph(), 'C-id')
+        original_cache = dict(merger._cached_trees)
+        tree = self.make_branch_and_memory_tree('tree')
+        merger.cache_trees_with_revision_ids([tree])
+        self.assertEqual(original_cache, merger._cached_trees)
+
+    def test_cache_trees_with_revision_ids_having_revision_id(self):
+        merger = self.make_Merger(self.setup_simple_graph(), 'C-id')
+        original_cache = dict(merger._cached_trees)
+        tree = merger.this_branch.repository.revision_tree('B-id')
+        original_cache['B-id'] = tree
+        merger.cache_trees_with_revision_ids([tree])
+        self.assertEqual(original_cache, merger._cached_trees)
+
     def test_find_base(self):
         merger = self.make_Merger(self.setup_simple_graph(), 'C-id')
         self.assertEqual('A-id', merger.base_rev_id)

=== modified file 'bzrlib/tests/test_merge_core.py'
--- bzrlib/tests/test_merge_core.py	2009-07-31 17:42:29 +0000
+++ bzrlib/tests/test_merge_core.py	2009-08-20 04:09:58 +0000
@@ -438,7 +438,6 @@
 
     def test_trivial_star_merge(self):
         """Test that merges in a star shape Just Work."""
-        self.thisFailsStrictLockCheck() # clone?
         # John starts a branch
         self.build_tree(("original/", "original/file1", "original/file2"))
         tree = self.make_branch_and_tree('original')
@@ -468,7 +467,6 @@
         self.assertEqual("Mary\n", open("original/file2", "rt").read())
 
     def test_conflicts(self):
-        self.thisFailsStrictLockCheck() # clone?
         os.mkdir('a')
         wta = self.make_branch_and_tree('a')
         a = wta.branch

=== modified file 'bzrlib/tests/test_msgeditor.py'
--- bzrlib/tests/test_msgeditor.py	2009-07-31 17:42:29 +0000
+++ bzrlib/tests/test_msgeditor.py	2009-08-20 04:09:58 +0000
@@ -81,7 +81,6 @@
 
     def make_multiple_pending_tree(self):
         from bzrlib import config
-        self.thisFailsStrictLockCheck() # clone?
         config.GlobalConfig().set_user_option('email',
                                               'Bilbo Baggins <bb@hobbit.net>')
         tree = self.make_branch_and_tree('a')

=== modified file 'bzrlib/tests/test_shelf.py'
--- bzrlib/tests/test_shelf.py	2009-08-18 12:49:22 +0000
+++ bzrlib/tests/test_shelf.py	2009-08-26 03:20:32 +0000
@@ -46,6 +46,8 @@
         tree.add(['foo'], ['foo-id'])
         tree.commit('foo')
         tree.rename_one('foo', 'bar')
+        tree.lock_tree_write()
+        self.addCleanup(tree.unlock)
         creator = shelf.ShelfCreator(tree, tree.basis_tree())
         self.addCleanup(creator.finalize)
         self.assertEqual([('rename', 'foo-id', 'foo', 'bar')],
@@ -61,13 +63,11 @@
                          shelf_trans_id))
 
     def test_shelve_rename(self):
-        self.thisFailsStrictLockCheck()
         creator = self.prepare_shelve_rename()
         creator.shelve_rename('foo-id')
         self.check_shelve_rename(creator)
 
     def test_shelve_change_handles_rename(self):
-        self.thisFailsStrictLockCheck()
         creator = self.prepare_shelve_rename()
         creator.shelve_change(('rename', 'foo-id', 'foo', 'bar'))
         self.check_shelve_rename(creator)
@@ -78,6 +78,8 @@
         tree.add(['foo', 'bar', 'foo/baz'], ['foo-id', 'bar-id', 'baz-id'])
         tree.commit('foo')
         tree.rename_one('foo/baz', 'bar/baz')
+        tree.lock_tree_write()
+        self.addCleanup(tree.unlock)
         creator = shelf.ShelfCreator(tree, tree.basis_tree())
         self.addCleanup(creator.finalize)
         self.assertEqual([('rename', 'baz-id', 'foo/baz', 'bar/baz')],
@@ -97,13 +99,11 @@
         self.assertEqual('foo/baz', tree.id2path('baz-id'))
 
     def test_shelve_move(self):
-        self.thisFailsStrictLockCheck()
         creator, tree = self.prepare_shelve_move()
         creator.shelve_rename('baz-id')
         self.check_shelve_move(creator, tree)
 
     def test_shelve_change_handles_move(self):
-        self.thisFailsStrictLockCheck()
         creator, tree = self.prepare_shelve_move()
         creator.shelve_change(('rename', 'baz-id', 'foo/baz', 'bar/baz'))
         self.check_shelve_move(creator, tree)
@@ -114,7 +114,6 @@
         self.assertFileEqual(expected_content, shelf_file)
 
     def prepare_content_change(self):
-        self.thisFailsStrictLockCheck()
         tree = self.make_branch_and_tree('.')
         tree.lock_write()
         self.addCleanup(tree.unlock)
@@ -177,7 +176,6 @@
             creator.shelf_transform.final_kind(s_bar_trans_id))
 
     def test_shelve_creation(self):
-        self.thisFailsStrictLockCheck()
         creator, tree = self.prepare_shelve_creation()
         creator.shelve_creation('foo-id')
         creator.shelve_creation('bar-id')
@@ -185,7 +183,6 @@
         self.check_shelve_creation(creator, tree)
 
     def test_shelve_change_handles_creation(self):
-        self.thisFailsStrictLockCheck()
         creator, tree = self.prepare_shelve_creation()
         creator.shelve_change(('add file', 'foo-id', 'file', 'foo'))
         creator.shelve_change(('add file', 'bar-id', 'directory', 'bar'))
@@ -218,17 +215,14 @@
         self.assertEqual(link_target, ptree.get_symlink_target('foo-id'))
 
     def test_shelve_symlink_creation(self):
-        self.thisFailsStrictLockCheck()
         self._test_shelve_symlink_creation('foo', 'bar')
 
     def test_shelve_unicode_symlink_creation(self):
         self.requireFeature(tests.UnicodeFilenameFeature)
-        self.thisFailsStrictLockCheck()
         self._test_shelve_symlink_creation(u'fo\N{Euro Sign}o',
                                            u'b\N{Euro Sign}ar')
 
     def test_shelve_change_handles_symlink_creation(self):
-        self.thisFailsStrictLockCheck()
         self._test_shelve_symlink_creation('foo', 'bar', shelve_change=True)
 
     def _test_shelve_symlink_target_change(self, link_name,
@@ -262,22 +256,18 @@
         self.assertEqual(new_target, ptree.get_symlink_target('foo-id'))
 
     def test_shelve_symlink_target_change(self):
-        self.thisFailsStrictLockCheck()
         self._test_shelve_symlink_target_change('foo', 'bar', 'baz')
 
     def test_shelve_unicode_symlink_target_change(self):
-        self.thisFailsStrictLockCheck()
         self.requireFeature(tests.UnicodeFilenameFeature)
         self._test_shelve_symlink_target_change(
             u'fo\N{Euro Sign}o', u'b\N{Euro Sign}ar', u'b\N{Euro Sign}az')
 
     def test_shelve_change_handles_symlink_target_change(self):
-        self.thisFailsStrictLockCheck()
         self._test_shelve_symlink_target_change('foo', 'bar', 'baz',
                                                 shelve_change=True)
 
     def test_shelve_creation_no_contents(self):
-        self.thisFailsStrictLockCheck()
         tree = self.make_branch_and_tree('.')
         tree.lock_write()
         self.addCleanup(tree.unlock)
@@ -322,7 +312,6 @@
         self.assertFileEqual('baz', 'tree/foo/bar')
 
     def test_shelve_deletion(self):
-        self.thisFailsStrictLockCheck()
         creator, tree = self.prepare_shelve_deletion()
         creator.shelve_deletion('foo-id')
         creator.shelve_deletion('bar-id')
@@ -330,7 +319,6 @@
         self.check_shelve_deletion(tree)
 
     def test_shelve_change_handles_deletion(self):
-        self.thisFailsStrictLockCheck()
         creator, tree = self.prepare_shelve_deletion()
         creator.shelve_change(('delete file', 'foo-id', 'directory', 'foo'))
         creator.shelve_change(('delete file', 'bar-id', 'file', 'foo/bar'))
@@ -338,12 +326,13 @@
         self.check_shelve_deletion(tree)
 
     def test_shelve_delete_contents(self):
-        self.thisFailsStrictLockCheck()
         tree = self.make_branch_and_tree('tree')
         self.build_tree(['tree/foo',])
         tree.add('foo', 'foo-id')
         tree.commit('Added file and directory')
         os.unlink('tree/foo')
+        tree.lock_tree_write()
+        self.addCleanup(tree.unlock)
         creator = shelf.ShelfCreator(tree, tree.basis_tree())
         self.addCleanup(creator.finalize)
         self.assertEqual([('delete file', 'foo-id', 'file', 'foo')],
@@ -359,6 +348,8 @@
         tree.commit('Added file and directory')
         os.unlink('tree/foo')
         os.mkdir('tree/foo')
+        tree.lock_tree_write()
+        self.addCleanup(tree.unlock)
         creator = shelf.ShelfCreator(tree, tree.basis_tree())
         self.addCleanup(creator.finalize)
         self.assertEqual([('change kind', 'foo-id', 'file', 'directory',
@@ -372,14 +363,12 @@
                          creator.shelf_transform._new_contents[s_trans_id])
 
     def test_shelve_change_kind(self):
-        self.thisFailsStrictLockCheck()
         creator = self.prepare_shelve_change_kind()
         creator.shelve_content_change('foo-id')
         creator.transform()
         self.check_shelve_change_kind(creator)
 
     def test_shelve_change_handles_change_kind(self):
-        self.thisFailsStrictLockCheck()
         creator = self.prepare_shelve_change_kind()
         creator.shelve_change(('change kind', 'foo-id', 'file', 'directory',
                                'foo'))
@@ -387,20 +376,22 @@
         self.check_shelve_change_kind(creator)
 
     def test_shelve_change_unknown_change(self):
-        self.thisFailsStrictLockCheck()
         tree = self.make_branch_and_tree('tree')
+        tree.lock_tree_write()
+        self.addCleanup(tree.unlock)
         creator = shelf.ShelfCreator(tree, tree.basis_tree())
         self.addCleanup(creator.finalize)
         e = self.assertRaises(ValueError, creator.shelve_change, ('unknown',))
         self.assertEqual('Unknown change kind: "unknown"', str(e))
 
     def test_shelve_unversion(self):
-        self.thisFailsStrictLockCheck()
         tree = self.make_branch_and_tree('tree')
         self.build_tree(['tree/foo',])
         tree.add('foo', 'foo-id')
         tree.commit('Added file and directory')
         tree.unversion(['foo-id'])
+        tree.lock_tree_write()
+        self.addCleanup(tree.unlock)
         creator = shelf.ShelfCreator(tree, tree.basis_tree())
         self.addCleanup(creator.finalize)
         self.assertEqual([('delete file', 'foo-id', 'file', 'foo')],
@@ -410,8 +401,9 @@
         self.failUnlessExists('tree/foo')
 
     def test_shelve_serialization(self):
-        self.thisFailsStrictLockCheck()
         tree = self.make_branch_and_tree('.')
+        tree.lock_tree_write()
+        self.addCleanup(tree.unlock)
         creator = shelf.ShelfCreator(tree, tree.basis_tree())
         self.addCleanup(creator.finalize)
         shelf_file = open('shelf', 'wb')
@@ -423,10 +415,11 @@
         self.assertFileEqual(EMPTY_SHELF, 'shelf')
 
     def test_write_shelf(self):
-        self.thisFailsStrictLockCheck()
         tree = self.make_branch_and_tree('tree')
         self.build_tree(['tree/foo'])
         tree.add('foo', 'foo-id')
+        tree.lock_tree_write()
+        self.addCleanup(tree.unlock)
         creator = shelf.ShelfCreator(tree, tree.basis_tree())
         self.addCleanup(creator.finalize)
         list(creator.iter_shelvable())
@@ -450,10 +443,13 @@
         tt.deserialize(records)
 
     def test_shelve_unversioned(self):
-        self.thisFailsStrictLockCheck()
         tree = self.make_branch_and_tree('tree')
-        self.assertRaises(errors.PathsNotVersionedError,
-                          shelf.ShelfCreator, tree, tree.basis_tree(), ['foo'])
+        tree.lock_tree_write()
+        try:
+            self.assertRaises(errors.PathsNotVersionedError,
+                              shelf.ShelfCreator, tree, tree.basis_tree(), ['foo'])
+        finally:
+            tree.unlock()
         # We should be able to lock/unlock the tree if ShelfCreator cleaned
         # after itself.
         wt = workingtree.WorkingTree.open('tree')
@@ -461,14 +457,26 @@
         wt.unlock()
         # And a second tentative should raise the same error (no
         # limbo/pending_deletion leftovers).
-        self.assertRaises(errors.PathsNotVersionedError,
-                          shelf.ShelfCreator, tree, tree.basis_tree(), ['foo'])
+        tree.lock_tree_write()
+        try:
+            self.assertRaises(errors.PathsNotVersionedError,
+                              shelf.ShelfCreator, tree, tree.basis_tree(), ['foo'])
+        finally:
+            tree.unlock()
+
+    def test_shelve_skips_added_root(self):
+        """Skip adds of the root when iterating through shelvable changes."""
+        tree = self.make_branch_and_tree('tree')
+        tree.lock_tree_write()
+        self.addCleanup(tree.unlock)
+        creator = shelf.ShelfCreator(tree, tree.basis_tree())
+        self.addCleanup(creator.finalize)
+        self.assertEqual([], list(creator.iter_shelvable()))
 
 
 class TestUnshelver(tests.TestCaseWithTransport):
 
     def test_make_merger(self):
-        self.thisFailsStrictLockCheck()
         tree = self.make_branch_and_tree('tree')
         tree.commit('first commit')
         self.build_tree_contents([('tree/foo', 'bar')])
@@ -491,7 +499,6 @@
             shelf_file.close()
 
     def test_unshelve_changed(self):
-        self.thisFailsStrictLockCheck()
         tree = self.make_branch_and_tree('tree')
         tree.lock_write()
         self.addCleanup(tree.unlock)
@@ -514,7 +521,6 @@
         self.assertFileEqual('z\na\nb\nd\n', 'tree/foo')
 
     def test_unshelve_deleted(self):
-        self.thisFailsStrictLockCheck()
         tree = self.make_branch_and_tree('tree')
         tree.lock_write()
         self.addCleanup(tree.unlock)
@@ -544,7 +550,6 @@
         self.assertFalse('bar-id' in tree)
 
     def test_unshelve_base(self):
-        self.thisFailsStrictLockCheck()
         tree = self.make_branch_and_tree('tree')
         tree.lock_write()
         self.addCleanup(tree.unlock)
@@ -565,15 +570,14 @@
         self.assertEqual('rev1', unshelver.base_tree.get_revision_id())
 
     def test_unshelve_serialization(self):
-        self.thisFailsStrictLockCheck()
         tree = self.make_branch_and_tree('.')
         self.build_tree_contents([('shelf', EMPTY_SHELF)])
         shelf_file = open('shelf', 'rb')
         self.addCleanup(shelf_file.close)
         unshelver = shelf.Unshelver.from_tree_and_shelf(tree, shelf_file)
+        unshelver.finalize()
 
     def test_corrupt_shelf(self):
-        self.thisFailsStrictLockCheck()
         tree = self.make_branch_and_tree('.')
         self.build_tree_contents([('shelf', EMPTY_SHELF.replace('metadata',
                                                                 'foo'))])
@@ -691,7 +695,10 @@
 
     def test_get_metadata(self):
         tree = self.make_branch_and_tree('.')
+        tree.lock_tree_write()
+        self.addCleanup(tree.unlock)
         creator = shelf.ShelfCreator(tree, tree.basis_tree())
+        self.addCleanup(creator.finalize)
         shelf_manager = tree.get_shelf_manager()
         shelf_id = shelf_manager.shelve_changes(creator, 'foo')
         metadata = shelf_manager.get_metadata(shelf_id)

=== modified file 'bzrlib/tests/test_shelf_ui.py'
--- bzrlib/tests/test_shelf_ui.py	2009-07-31 17:42:29 +0000
+++ bzrlib/tests/test_shelf_ui.py	2009-08-26 03:20:32 +0000
@@ -19,7 +19,12 @@
 import os
 import sys
 
-from bzrlib import errors, shelf_ui, tests
+from bzrlib import (
+    errors,
+    shelf_ui,
+    revision,
+    tests,
+)
 
 
 class ExpectShelver(shelf_ui.Shelver):
@@ -67,23 +72,26 @@
         return tree
 
     def test_unexpected_prompt_failure(self):
-        self.thisFailsStrictLockCheck()
         tree = self.create_shelvable_tree()
+        tree.lock_tree_write()
+        self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree())
         e = self.assertRaises(AssertionError, shelver.run)
         self.assertEqual('Unexpected prompt: Shelve? [yNfq?]', str(e))
 
     def test_wrong_prompt_failure(self):
-        self.thisFailsStrictLockCheck()
         tree = self.create_shelvable_tree()
+        tree.lock_tree_write()
+        self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree())
         shelver.expect('foo', 'y')
         e = self.assertRaises(AssertionError, shelver.run)
         self.assertEqual('Wrong prompt: Shelve? [yNfq?]', str(e))
 
     def test_shelve_not_diff(self):
-        self.thisFailsStrictLockCheck()
         tree = self.create_shelvable_tree()
+        tree.lock_tree_write()
+        self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree())
         shelver.expect('Shelve? [yNfq?]', 'n')
         shelver.expect('Shelve? [yNfq?]', 'n')
@@ -92,8 +100,9 @@
         self.assertFileEqual(LINES_ZY, 'tree/foo')
 
     def test_shelve_diff_no(self):
-        self.thisFailsStrictLockCheck()
         tree = self.create_shelvable_tree()
+        tree.lock_tree_write()
+        self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree())
         shelver.expect('Shelve? [yNfq?]', 'y')
         shelver.expect('Shelve? [yNfq?]', 'y')
@@ -102,8 +111,9 @@
         self.assertFileEqual(LINES_ZY, 'tree/foo')
 
     def test_shelve_diff(self):
-        self.thisFailsStrictLockCheck()
         tree = self.create_shelvable_tree()
+        tree.lock_tree_write()
+        self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree())
         shelver.expect('Shelve? [yNfq?]', 'y')
         shelver.expect('Shelve? [yNfq?]', 'y')
@@ -112,8 +122,9 @@
         self.assertFileEqual(LINES_AJ, 'tree/foo')
 
     def test_shelve_one_diff(self):
-        self.thisFailsStrictLockCheck()
         tree = self.create_shelvable_tree()
+        tree.lock_tree_write()
+        self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree())
         shelver.expect('Shelve? [yNfq?]', 'y')
         shelver.expect('Shelve? [yNfq?]', 'n')
@@ -122,9 +133,10 @@
         self.assertFileEqual(LINES_AY, 'tree/foo')
 
     def test_shelve_binary_change(self):
-        self.thisFailsStrictLockCheck()
         tree = self.create_shelvable_tree()
         self.build_tree_contents([('tree/foo', '\x00')])
+        tree.lock_tree_write()
+        self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree())
         shelver.expect('Shelve binary changes? [yNfq?]', 'y')
         shelver.expect('Shelve 1 change(s)? [yNfq?]', 'y')
@@ -132,9 +144,10 @@
         self.assertFileEqual(LINES_AJ, 'tree/foo')
 
     def test_shelve_rename(self):
-        self.thisFailsStrictLockCheck()
         tree = self.create_shelvable_tree()
         tree.rename_one('foo', 'bar')
+        tree.lock_tree_write()
+        self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree())
         shelver.expect('Shelve renaming "foo" => "bar"? [yNfq?]', 'y')
         shelver.expect('Shelve? [yNfq?]', 'y')
@@ -144,9 +157,10 @@
         self.assertFileEqual(LINES_AJ, 'tree/foo')
 
     def test_shelve_deletion(self):
-        self.thisFailsStrictLockCheck()
         tree = self.create_shelvable_tree()
         os.unlink('tree/foo')
+        tree.lock_tree_write()
+        self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree())
         shelver.expect('Shelve removing file "foo"? [yNfq?]', 'y')
         shelver.expect('Shelve 1 change(s)? [yNfq?]', 'y')
@@ -154,11 +168,12 @@
         self.assertFileEqual(LINES_AJ, 'tree/foo')
 
     def test_shelve_creation(self):
-        self.thisFailsStrictLockCheck()
         tree = self.make_branch_and_tree('tree')
         tree.commit('add tree root')
         self.build_tree(['tree/foo'])
         tree.add('foo')
+        tree.lock_tree_write()
+        self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree())
         shelver.expect('Shelve adding file "foo"? [yNfq?]', 'y')
         shelver.expect('Shelve 1 change(s)? [yNfq?]', 'y')
@@ -166,17 +181,17 @@
         self.failIfExists('tree/foo')
 
     def test_shelve_kind_change(self):
-        self.thisFailsStrictLockCheck()
         tree = self.create_shelvable_tree()
         os.unlink('tree/foo')
         os.mkdir('tree/foo')
+        tree.lock_tree_write()
+        self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree())
         shelver.expect('Shelve changing "foo" from file to directory? [yNfq?]',
                        'y')
         shelver.expect('Shelve 1 change(s)? [yNfq?]', 'y')
 
     def test_shelve_modify_target(self):
-        self.thisFailsStrictLockCheck()
         self.requireFeature(tests.SymlinkFeature)
         tree = self.create_shelvable_tree()
         os.symlink('bar', 'tree/baz')
@@ -184,6 +199,8 @@
         tree.commit("Add symlink")
         os.unlink('tree/baz')
         os.symlink('vax', 'tree/baz')
+        tree.lock_tree_write()
+        self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree())
         shelver.expect('Shelve changing target of "baz" from "bar" to '
                 '"vax"? [yNfq?]', 'y')
@@ -192,8 +209,9 @@
         self.assertEqual('bar', os.readlink('tree/baz'))
 
     def test_shelve_finish(self):
-        self.thisFailsStrictLockCheck()
         tree = self.create_shelvable_tree()
+        tree.lock_tree_write()
+        self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree())
         shelver.expect('Shelve? [yNfq?]', 'f')
         shelver.expect('Shelve 2 change(s)? [yNfq?]', 'y')
@@ -201,32 +219,39 @@
         self.assertFileEqual(LINES_AJ, 'tree/foo')
 
     def test_shelve_quit(self):
-        self.thisFailsStrictLockCheck()
         tree = self.create_shelvable_tree()
+        tree.lock_tree_write()
+        self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree())
         shelver.expect('Shelve? [yNfq?]', 'q')
         self.assertRaises(errors.UserAbort, shelver.run)
         self.assertFileEqual(LINES_ZY, 'tree/foo')
 
     def test_shelve_all(self):
-        self.thisFailsStrictLockCheck()
         tree = self.create_shelvable_tree()
-        ExpectShelver.from_args(sys.stdout, all=True, directory='tree').run()
+        shelver = ExpectShelver.from_args(sys.stdout, all=True,
+            directory='tree')
+        try:
+            shelver.run()
+        finally:
+            shelver.work_tree.unlock()
         self.assertFileEqual(LINES_AJ, 'tree/foo')
 
     def test_shelve_filename(self):
-        self.thisFailsStrictLockCheck()
         tree = self.create_shelvable_tree()
         self.build_tree(['tree/bar'])
         tree.add('bar')
+        tree.lock_tree_write()
+        self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree(), file_list=['bar'])
         shelver.expect('Shelve adding file "bar"? [yNfq?]', 'y')
         shelver.expect('Shelve 1 change(s)? [yNfq?]', 'y')
         shelver.run()
 
     def test_shelve_help(self):
-        self.thisFailsStrictLockCheck()
         tree = self.create_shelvable_tree()
+        tree.lock_tree_write()
+        self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree())
         shelver.expect('Shelve? [yNfq?]', '?')
         shelver.expect('Shelve? [(y)es, (N)o, (f)inish, or (q)uit]', 'f')
@@ -234,20 +259,61 @@
         shelver.run()
 
     def test_shelve_distroy(self):
-        self.thisFailsStrictLockCheck()
         tree = self.create_shelvable_tree()
         shelver = shelf_ui.Shelver.from_args(sys.stdout, all=True,
                                              directory='tree', destroy=True)
-        shelver.run()
+        try:
+            shelver.run()
+        finally:
+            shelver.work_tree.unlock()
         self.assertIs(None, tree.get_shelf_manager().last_shelf())
         self.assertFileEqual(LINES_AJ, 'tree/foo')
 
+    @staticmethod
+    def shelve_all(tree, target_revision_id):
+        tree.lock_write()
+        try:
+            target = tree.branch.repository.revision_tree(target_revision_id)
+            shelver = shelf_ui.Shelver(tree, target, auto=True,
+                                       auto_apply=True)
+            shelver.run()
+        finally:
+            tree.unlock()
+
+    def test_shelve_old_root_deleted(self):
+        tree1 = self.make_branch_and_tree('tree1')
+        tree1.commit('add root')
+        tree2 = self.make_branch_and_tree('tree2')
+        rev2 = tree2.commit('add root')
+        tree1.merge_from_branch(tree2.branch,
+                                from_revision=revision.NULL_REVISION)
+        tree1.commit('Replaced root entry')
+        # This is essentially assertNotRaises(InconsistentDelta)
+        self.expectFailure('Cannot shelve replacing a root entry',
+                           self.assertRaises, AssertionError,
+                           self.assertRaises, errors.InconsistentDelta,
+                           self.shelve_all, tree1, rev2)
+
+    def test_shelve_split(self):
+        outer_tree = self.make_branch_and_tree('outer')
+        outer_tree.commit('Add root')
+        inner_tree = self.make_branch_and_tree('outer/inner')
+        rev2 = inner_tree.commit('Add root')
+        outer_tree.subsume(inner_tree)
+        # This is essentially assertNotRaises(ValueError).
+        # The ValueError is 'None is not a valid file id'.
+        self.expectFailure('Cannot shelve a join back to the inner tree.',
+                           self.assertRaises, AssertionError,
+                           self.assertRaises, ValueError, self.shelve_all,
+                           outer_tree, rev2)
+
 
 class TestApplyReporter(TestShelver):
 
     def test_shelve_not_diff(self):
-        self.thisFailsStrictLockCheck()
         tree = self.create_shelvable_tree()
+        tree.lock_tree_write()
+        self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree(),
                                 reporter=shelf_ui.ApplyReporter())
         shelver.expect('Apply change? [yNfq?]', 'n')
@@ -257,8 +323,9 @@
         self.assertFileEqual(LINES_ZY, 'tree/foo')
 
     def test_shelve_diff_no(self):
-        self.thisFailsStrictLockCheck()
         tree = self.create_shelvable_tree()
+        tree.lock_tree_write()
+        self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree(),
                                 reporter=shelf_ui.ApplyReporter())
         shelver.expect('Apply change? [yNfq?]', 'y')
@@ -268,8 +335,9 @@
         self.assertFileEqual(LINES_ZY, 'tree/foo')
 
     def test_shelve_diff(self):
-        self.thisFailsStrictLockCheck()
         tree = self.create_shelvable_tree()
+        tree.lock_tree_write()
+        self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree(),
                                 reporter=shelf_ui.ApplyReporter())
         shelver.expect('Apply change? [yNfq?]', 'y')
@@ -279,9 +347,10 @@
         self.assertFileEqual(LINES_AJ, 'tree/foo')
 
     def test_shelve_binary_change(self):
-        self.thisFailsStrictLockCheck()
         tree = self.create_shelvable_tree()
         self.build_tree_contents([('tree/foo', '\x00')])
+        tree.lock_tree_write()
+        self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree(),
                                 reporter=shelf_ui.ApplyReporter())
         shelver.expect('Apply binary changes? [yNfq?]', 'y')
@@ -290,9 +359,10 @@
         self.assertFileEqual(LINES_AJ, 'tree/foo')
 
     def test_shelve_rename(self):
-        self.thisFailsStrictLockCheck()
         tree = self.create_shelvable_tree()
         tree.rename_one('foo', 'bar')
+        tree.lock_tree_write()
+        self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree(),
                                 reporter=shelf_ui.ApplyReporter())
         shelver.expect('Rename "bar" => "foo"? [yNfq?]', 'y')
@@ -303,9 +373,10 @@
         self.assertFileEqual(LINES_AJ, 'tree/foo')
 
     def test_shelve_deletion(self):
-        self.thisFailsStrictLockCheck()
         tree = self.create_shelvable_tree()
         os.unlink('tree/foo')
+        tree.lock_tree_write()
+        self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree(),
                                 reporter=shelf_ui.ApplyReporter())
         shelver.expect('Add file "foo"? [yNfq?]', 'y')
@@ -314,11 +385,12 @@
         self.assertFileEqual(LINES_AJ, 'tree/foo')
 
     def test_shelve_creation(self):
-        self.thisFailsStrictLockCheck()
         tree = self.make_branch_and_tree('tree')
         tree.commit('add tree root')
         self.build_tree(['tree/foo'])
         tree.add('foo')
+        tree.lock_tree_write()
+        self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree(),
                                 reporter=shelf_ui.ApplyReporter())
         shelver.expect('Delete file "foo"? [yNfq?]', 'y')
@@ -327,17 +399,17 @@
         self.failIfExists('tree/foo')
 
     def test_shelve_kind_change(self):
-        self.thisFailsStrictLockCheck()
         tree = self.create_shelvable_tree()
         os.unlink('tree/foo')
         os.mkdir('tree/foo')
+        tree.lock_tree_write()
+        self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree(),
                                reporter=shelf_ui.ApplyReporter())
         shelver.expect('Change "foo" from directory to a file? [yNfq?]', 'y')
         shelver.expect('Apply 1 change(s)? [yNfq?]', 'y')
 
     def test_shelve_modify_target(self):
-        self.thisFailsStrictLockCheck()
         self.requireFeature(tests.SymlinkFeature)
         tree = self.create_shelvable_tree()
         os.symlink('bar', 'tree/baz')
@@ -345,6 +417,8 @@
         tree.commit("Add symlink")
         os.unlink('tree/baz')
         os.symlink('vax', 'tree/baz')
+        tree.lock_tree_write()
+        self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree(),
                                 reporter=shelf_ui.ApplyReporter())
         shelver.expect('Change target of "baz" from "vax" to "bar"? [yNfq?]',
@@ -358,16 +432,19 @@
 
     def create_tree_with_shelf(self):
         tree = self.make_branch_and_tree('tree')
-        self.build_tree_contents([('tree/foo', LINES_AJ)])
-        tree.add('foo', 'foo-id')
-        tree.commit('added foo')
-        self.build_tree_contents([('tree/foo', LINES_ZY)])
-        shelf_ui.Shelver(tree, tree.basis_tree(), auto_apply=True,
-                         auto=True).run()
+        tree.lock_write()
+        try:
+            self.build_tree_contents([('tree/foo', LINES_AJ)])
+            tree.add('foo', 'foo-id')
+            tree.commit('added foo')
+            self.build_tree_contents([('tree/foo', LINES_ZY)])
+            shelf_ui.Shelver(tree, tree.basis_tree(), auto_apply=True,
+                             auto=True).run()
+        finally:
+            tree.unlock()
         return tree
 
     def test_unshelve(self):
-        self.thisFailsStrictLockCheck()
         tree = self.create_tree_with_shelf()
         tree.lock_write()
         self.addCleanup(tree.unlock)
@@ -376,21 +453,27 @@
         self.assertFileEqual(LINES_ZY, 'tree/foo')
 
     def test_unshelve_args(self):
-        self.thisFailsStrictLockCheck()
         tree = self.create_tree_with_shelf()
-        shelf_ui.Unshelver.from_args(directory='tree').run()
+        unshelver = shelf_ui.Unshelver.from_args(directory='tree')
+        try:
+            unshelver.run()
+        finally:
+            unshelver.tree.unlock()
         self.assertFileEqual(LINES_ZY, 'tree/foo')
         self.assertIs(None, tree.get_shelf_manager().last_shelf())
 
     def test_unshelve_args_dry_run(self):
-        self.thisFailsStrictLockCheck()
         tree = self.create_tree_with_shelf()
-        shelf_ui.Unshelver.from_args(directory='tree', action='dry-run').run()
+        unshelver = shelf_ui.Unshelver.from_args(directory='tree',
+            action='dry-run')
+        try:
+            unshelver.run()
+        finally:
+            unshelver.tree.unlock()
         self.assertFileEqual(LINES_AJ, 'tree/foo')
         self.assertEqual(1, tree.get_shelf_manager().last_shelf())
 
     def test_unshelve_args_delete_only(self):
-        self.thisFailsStrictLockCheck()
         tree = self.make_branch_and_tree('tree')
         manager = tree.get_shelf_manager()
         shelf_file = manager.new_shelf()[1]
@@ -400,11 +483,13 @@
             shelf_file.close()
         unshelver = shelf_ui.Unshelver.from_args(directory='tree',
                                                  action='delete-only')
-        unshelver.run()
+        try:
+            unshelver.run()
+        finally:
+            unshelver.tree.unlock()
         self.assertIs(None, manager.last_shelf())
 
     def test_unshelve_args_invalid_shelf_id(self):
-        self.thisFailsStrictLockCheck()
         tree = self.make_branch_and_tree('tree')
         manager = tree.get_shelf_manager()
         shelf_file = manager.new_shelf()[1]

=== modified file 'bzrlib/tests/test_status.py'
--- bzrlib/tests/test_status.py	2009-07-31 17:42:29 +0000
+++ bzrlib/tests/test_status.py	2009-08-20 04:09:58 +0000
@@ -43,7 +43,6 @@
         self.assertContainsRe(output.getvalue(), 'empty commit')
 
     def make_multiple_pending_tree(self):
-        self.thisFailsStrictLockCheck() # clone?
         config.GlobalConfig().set_user_option('email', 'Joe Foo <joe@foo.com>')
         tree = self.make_branch_and_tree('a')
         tree.commit('commit 1', timestamp=1196796819, timezone=0)
@@ -100,7 +99,6 @@
 
     def test_pending_with_ghosts(self):
         """Test when a pending merge's ancestry includes ghosts."""
-        self.thisFailsStrictLockCheck() # clone?
         config.GlobalConfig().set_user_option('email', 'Joe Foo <joe@foo.com>')
         tree = self.make_branch_and_tree('a')
         tree.commit('empty commit')

=== modified file 'bzrlib/tests/test_tree.py'
--- bzrlib/tests/test_tree.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_tree.py	2009-08-26 03:20:32 +0000
@@ -123,7 +123,7 @@
         wt = self.make_branch_and_tree('.')
         delta = wt.changes_from(wt.basis_tree())
         self.assertEqual(len(delta.added), 0)
-        delta = wt.changes_from(wt.basis_tree(), wt, include_root=True)
+        delta = wt.changes_from(wt.basis_tree(), include_root=True)
         self.assertEqual(len(delta.added), 1)
         self.assertEqual(delta.added[0][0], '')
 
@@ -139,7 +139,7 @@
 
         # we need to pass a known file with an unknown file to get this to
         # fail when expected.
-        delta = wt.changes_from(wt.basis_tree(), wt,
+        delta = wt.changes_from(wt.basis_tree(),
             specific_files=['known_file', 'unknown_file'] ,
             require_versioned=False)
         self.assertEqual(len(delta.added), 1)

=== modified file 'bzrlib/tests/test_workingtree_4.py'
--- bzrlib/tests/test_workingtree_4.py	2009-07-01 07:26:30 +0000
+++ bzrlib/tests/test_workingtree_4.py	2009-08-26 03:20:32 +0000
@@ -490,7 +490,8 @@
         subtree = self.make_branch_and_tree('dir')
         # the most primitive operation: kind
         self.assertEqual('directory', tree.kind('dir-id'))
-        # a diff against the basis should give us a directory
+        # a diff against the basis should give us a directory and the root (as
+        # the root is new too).
         tree.lock_read()
         expected = [('dir-id',
             (None, u'dir'),
@@ -499,7 +500,9 @@
             (None, 'root'),
             (None, u'dir'),
             (None, 'directory'),
-            (None, False))]
+            (None, False)),
+            ('root', (None, u''), True, (False, True), (None, None),
+            (None, u''), (None, 'directory'), (None, 0))]
         self.assertEqual(expected, list(tree.iter_changes(tree.basis_tree(),
             specific_files=['dir'])))
         tree.unlock()

=== modified file 'bzrlib/transform.py'
--- bzrlib/transform.py	2009-08-04 11:40:59 +0000
+++ bzrlib/transform.py	2009-08-26 03:20:32 +0000
@@ -1790,7 +1790,8 @@
             if self._transform.final_file_id(trans_id) is None:
                 yield self._final_paths._determine_path(trans_id)
 
-    def _make_inv_entries(self, ordered_entries, specific_file_ids=None):
+    def _make_inv_entries(self, ordered_entries, specific_file_ids=None,
+        yield_parents=False):
         for trans_id, parent_file_id in ordered_entries:
             file_id = self._transform.final_file_id(trans_id)
             if file_id is None:
@@ -1822,7 +1823,7 @@
                 ordered_ids.append((trans_id, parent_file_id))
         return ordered_ids
 
-    def iter_entries_by_dir(self, specific_file_ids=None):
+    def iter_entries_by_dir(self, specific_file_ids=None, yield_parents=False):
         # This may not be a maximally efficient implementation, but it is
         # reasonably straightforward.  An implementation that grafts the
         # TreeTransform changes onto the tree's iter_entries_by_dir results
@@ -1830,7 +1831,7 @@
         # position.
         ordered_ids = self._list_files_by_dir()
         for entry, trans_id in self._make_inv_entries(ordered_ids,
-                                                      specific_file_ids):
+            specific_file_ids, yield_parents=yield_parents):
             yield unicode(self._final_paths.get_path(trans_id)), entry
 
     def _iter_entries_for_dir(self, dir_path):

=== modified file 'bzrlib/tree.py'
--- bzrlib/tree.py	2009-07-17 06:04:35 +0000
+++ bzrlib/tree.py	2009-08-26 03:20:32 +0000
@@ -170,7 +170,7 @@
         return self.bzrdir.is_control_filename(filename)
 
     @needs_read_lock
-    def iter_entries_by_dir(self, specific_file_ids=None):
+    def iter_entries_by_dir(self, specific_file_ids=None, yield_parents=False):
         """Walk the tree in 'by_dir' order.
 
         This will yield each entry in the tree as a (path, entry) tuple.
@@ -193,9 +193,13 @@
 
         The yield order (ignoring root) would be::
           a, f, a/b, a/d, a/b/c, a/d/e, f/g
+
+        :param yield_parents: If True, yield the parents from the root leading
+            down to specific_file_ids that have been requested. This has no
+            impact if specific_file_ids is None.
         """
         return self.inventory.iter_entries_by_dir(
-            specific_file_ids=specific_file_ids)
+            specific_file_ids=specific_file_ids, yield_parents=yield_parents)
 
     def iter_references(self):
         if self.supports_tree_reference():
@@ -846,6 +850,86 @@
 
     _optimisers = []
 
+    def _changes_from_entries(self, source_entry, target_entry,
+        source_path=None, target_path=None):
+        """Generate a iter_changes tuple between source_entry and target_entry.
+
+        :param source_entry: An inventory entry from self.source, or None.
+        :param target_entry: An inventory entry from self.target, or None.
+        :param source_path: The path of source_entry, if known. If not known
+            it will be looked up.
+        :param target_path: The path of target_entry, if known. If not known
+            it will be looked up.
+        :return: A tuple, item 0 of which is an iter_changes result tuple, and
+            item 1 is True if there are any changes in the result tuple.
+        """
+        if source_entry is None:
+            if target_entry is None:
+                return None
+            file_id = target_entry.file_id
+        else:
+            file_id = source_entry.file_id
+        if source_entry is not None:
+            source_versioned = True
+            source_name = source_entry.name
+            source_parent = source_entry.parent_id
+            if source_path is None:
+                source_path = self.source.id2path(file_id)
+            source_kind, source_executable, source_stat = \
+                self.source._comparison_data(source_entry, source_path)
+        else:
+            source_versioned = False
+            source_name = None
+            source_parent = None
+            source_kind = None
+            source_executable = None
+        if target_entry is not None:
+            target_versioned = True
+            target_name = target_entry.name
+            target_parent = target_entry.parent_id
+            if target_path is None:
+                target_path = self.target.id2path(file_id)
+            target_kind, target_executable, target_stat = \
+                self.target._comparison_data(target_entry, target_path)
+        else:
+            target_versioned = False
+            target_name = None
+            target_parent = None
+            target_kind = None
+            target_executable = None
+        versioned = (source_versioned, target_versioned)
+        kind = (source_kind, target_kind)
+        changed_content = False
+        if source_kind != target_kind:
+            changed_content = True
+        elif source_kind == 'file':
+            if (self.source.get_file_sha1(file_id, source_path, source_stat) !=
+                self.target.get_file_sha1(file_id, target_path, target_stat)):
+                changed_content = True
+        elif source_kind == 'symlink':
+            if (self.source.get_symlink_target(file_id) !=
+                self.target.get_symlink_target(file_id)):
+                changed_content = True
+            # XXX: Yes, the indentation below is wrong. But fixing it broke
+            # test_merge.TestMergerEntriesLCAOnDisk.
+            # test_nested_tree_subtree_renamed_and_modified. We'll wait for
+            # the fix from bzr.dev -- vila 2009026
+            elif source_kind == 'tree-reference':
+                if (self.source.get_reference_revision(file_id, source_path)
+                    != self.target.get_reference_revision(file_id, target_path)):
+                    changed_content = True
+        parent = (source_parent, target_parent)
+        name = (source_name, target_name)
+        executable = (source_executable, target_executable)
+        if (changed_content is not False or versioned[0] != versioned[1]
+            or parent[0] != parent[1] or name[0] != name[1] or
+            executable[0] != executable[1]):
+            changes = True
+        else:
+            changes = False
+        return (file_id, (source_path, target_path), changed_content,
+                versioned, parent, name, kind, executable), changes
+
     @needs_read_lock
     def compare(self, want_unchanged=False, specific_files=None,
         extra_trees=None, require_versioned=False, include_root=False,
@@ -914,6 +998,12 @@
         :param require_versioned: Raise errors.PathsNotVersionedError if a
             path in the specific_files list is not versioned in one of
             source, target or extra_trees.
+        :param specific_files: An optional list of file paths to restrict the
+            comparison to. When mapping filenames to ids, all matches in all
+            trees (including optional extra_trees) are used, and all children
+            of matched directories are included. The parents in the target tree
+            of the specific files up to and including the root of the tree are
+            always evaluated for changes too.
         :param want_unversioned: Should unversioned files be returned in the
             output. An unversioned file is defined as one with (False, False)
             for the versioned pair.
@@ -921,11 +1011,23 @@
         lookup_trees = [self.source]
         if extra_trees:
              lookup_trees.extend(extra_trees)
+        # The ids of items we need to examine to insure delta consistency.
+        precise_file_ids = set()
+        changed_file_ids = []
         if specific_files == []:
             specific_file_ids = []
         else:
             specific_file_ids = self.target.paths2ids(specific_files,
                 lookup_trees, require_versioned=require_versioned)
+        if specific_files is not None:
+            # reparented or added entries must have their parents included
+            # so that valid deltas can be created. The seen_parents set
+            # tracks the parents that we need to have.
+            # The seen_dirs set tracks directory entries we've yielded.
+            # After outputting version object in to_entries we set difference
+            # the two seen sets and start checking parents.
+            seen_parents = set()
+            seen_dirs = set()
         if want_unversioned:
             all_unversioned = sorted([(p.split('/'), p) for p in
                                      self.target.extras()
@@ -946,68 +1048,43 @@
         # can be extras. So the fake_entry is solely used to look up
         # executable it values when execute is not supported.
         fake_entry = InventoryFile('unused', 'unused', 'unused')
-        for to_path, to_entry in to_entries_by_dir:
-            while all_unversioned and all_unversioned[0][0] < to_path.split('/'):
+        for target_path, target_entry in to_entries_by_dir:
+            while (all_unversioned and
+                all_unversioned[0][0] < target_path.split('/')):
                 unversioned_path = all_unversioned.popleft()
-                to_kind, to_executable, to_stat = \
+                target_kind, target_executable, target_stat = \
                     self.target._comparison_data(fake_entry, unversioned_path[1])
                 yield (None, (None, unversioned_path[1]), True, (False, False),
                     (None, None),
                     (None, unversioned_path[0][-1]),
-                    (None, to_kind),
-                    (None, to_executable))
-            file_id = to_entry.file_id
-            to_paths[file_id] = to_path
+                    (None, target_kind),
+                    (None, target_executable))
+            source_path, source_entry = from_data.get(target_entry.file_id,
+                (None, None))
+            result, changes = self._changes_from_entries(source_entry,
+                target_entry, source_path=source_path, target_path=target_path)
+            to_paths[result[0]] = result[1][1]
             entry_count += 1
-            changed_content = False
-            from_path, from_entry = from_data.get(file_id, (None, None))
-            from_versioned = (from_entry is not None)
-            if from_entry is not None:
-                from_versioned = True
-                from_name = from_entry.name
-                from_parent = from_entry.parent_id
-                from_kind, from_executable, from_stat = \
-                    self.source._comparison_data(from_entry, from_path)
+            if result[3][0]:
                 entry_count += 1
-            else:
-                from_versioned = False
-                from_kind = None
-                from_parent = None
-                from_name = None
-                from_executable = None
-            versioned = (from_versioned, True)
-            to_kind, to_executable, to_stat = \
-                self.target._comparison_data(to_entry, to_path)
-            kind = (from_kind, to_kind)
-            if kind[0] != kind[1]:
-                changed_content = True
-            elif from_kind == 'file':
-                if (self.source.get_file_sha1(file_id, from_path, from_stat) !=
-                    self.target.get_file_sha1(file_id, to_path, to_stat)):
-                    changed_content = True
-            elif from_kind == 'symlink':
-                if (self.source.get_symlink_target(file_id) !=
-                    self.target.get_symlink_target(file_id)):
-                    changed_content = True
-                # XXX: Yes, the indentation below is wrong. But fixing it broke
-                # test_merge.TestMergerEntriesLCAOnDisk.
-                # test_nested_tree_subtree_renamed_and_modified. We'll wait for
-                # the fix from bzr.dev -- vila 2009026
-                elif from_kind == 'tree-reference':
-                    if (self.source.get_reference_revision(file_id, from_path)
-                        != self.target.get_reference_revision(file_id, to_path)):
-                        changed_content = True
-            parent = (from_parent, to_entry.parent_id)
-            name = (from_name, to_entry.name)
-            executable = (from_executable, to_executable)
             if pb is not None:
                 pb.update('comparing files', entry_count, num_entries)
-            if (changed_content is not False or versioned[0] != versioned[1]
-                or parent[0] != parent[1] or name[0] != name[1] or
-                executable[0] != executable[1] or include_unchanged):
-                yield (file_id, (from_path, to_path), changed_content,
-                    versioned, parent, name, kind, executable)
-
+            if changes or include_unchanged:
+                if specific_file_ids is not None:
+                    new_parent_id = result[4][1]
+                    precise_file_ids.add(new_parent_id)
+                    changed_file_ids.append(result[0])
+                yield result
+            # Ensure correct behaviour for reparented/added specific files.
+            if specific_files is not None:
+                # Record output dirs
+                if result[6][1] == 'directory':
+                    seen_dirs.add(result[0])
+                # Record parents of reparented/added entries.
+                versioned = result[3]
+                parents = result[4]
+                if not versioned[0] or parents[0] != parents[1]:
+                    seen_parents.add(parents[1])
         while all_unversioned:
             # yield any trailing unversioned paths
             unversioned_path = all_unversioned.popleft()
@@ -1018,30 +1095,18 @@
                 (None, unversioned_path[0][-1]),
                 (None, to_kind),
                 (None, to_executable))
-
-        def get_to_path(to_entry):
-            if to_entry.parent_id is None:
-                to_path = '' # the root
-            else:
-                if to_entry.parent_id not in to_paths:
-                    # recurse up
-                    return get_to_path(self.target.inventory[to_entry.parent_id])
-                to_path = osutils.pathjoin(to_paths[to_entry.parent_id],
-                                           to_entry.name)
-            to_paths[to_entry.file_id] = to_path
-            return to_path
-
+        # Yield all remaining source paths
         for path, from_entry in from_entries_by_dir:
             file_id = from_entry.file_id
             if file_id in to_paths:
                 # already returned
                 continue
-            if not file_id in self.target.all_file_ids():
+            if file_id not in self.target.all_file_ids():
                 # common case - paths we have not emitted are not present in
                 # target.
                 to_path = None
             else:
-                to_path = get_to_path(self.target.inventory[file_id])
+                to_path = self.target.id2path(file_id)
             entry_count += 1
             if pb is not None:
                 pb.update('comparing files', entry_count, num_entries)
@@ -1054,8 +1119,111 @@
             executable = (from_executable, None)
             changed_content = from_kind is not None
             # the parent's path is necessarily known at this point.
+            changed_file_ids.append(file_id)
             yield(file_id, (path, to_path), changed_content, versioned, parent,
                   name, kind, executable)
+        changed_file_ids = set(changed_file_ids)
+        if specific_file_ids is not None:
+            for result in self._handle_precise_ids(precise_file_ids,
+                changed_file_ids):
+                yield result
+
+    def _get_entry(self, tree, file_id):
+        """Get an inventory entry from a tree, with missing entries as None.
+
+        If the tree raises NotImplementedError on accessing .inventory, then
+        this is worked around using iter_entries_by_dir on just the file id
+        desired.
+
+        :param tree: The tree to lookup the entry in.
+        :param file_id: The file_id to lookup.
+        """
+        try:
+            inventory = tree.inventory
+        except NotImplementedError:
+            # No inventory available.
+            try:
+                iterator = tree.iter_entries_by_dir(specific_file_ids=[file_id])
+                return iterator.next()[1]
+            except StopIteration:
+                return None
+        else:
+            try:
+                return inventory[file_id]
+            except errors.NoSuchId:
+                return None
+
+    def _handle_precise_ids(self, precise_file_ids, changed_file_ids,
+        discarded_changes=None):
+        """Fill out a partial iter_changes to be consistent.
+
+        :param precise_file_ids: The file ids of parents that were seen during
+            the iter_changes.
+        :param changed_file_ids: The file ids of already emitted items.
+        :param discarded_changes: An optional dict of precalculated
+            iter_changes items which the partial iter_changes had not output
+            but had calculated.
+        :return: A generator of iter_changes items to output.
+        """
+        # process parents of things that had changed under the users
+        # requested paths to prevent incorrect paths or parent ids which
+        # aren't in the tree.
+        while precise_file_ids:
+            precise_file_ids.discard(None)
+            # Don't emit file_ids twice
+            precise_file_ids.difference_update(changed_file_ids)
+            if not precise_file_ids:
+                break
+            # If the there was something at a given output path in source, we
+            # have to include the entry from source in the delta, or we would
+            # be putting this entry into a used path.
+            paths = []
+            for parent_id in precise_file_ids:
+                try:
+                    paths.append(self.target.id2path(parent_id))
+                except errors.NoSuchId:
+                    # This id has been dragged in from the source by delta
+                    # expansion and isn't present in target at all: we don't
+                    # need to check for path collisions on it.
+                    pass
+            for path in paths:
+                old_id = self.source.path2id(path)
+                precise_file_ids.add(old_id)
+            precise_file_ids.discard(None)
+            current_ids = precise_file_ids
+            precise_file_ids = set()
+            # We have to emit all of precise_file_ids that have been altered.
+            # We may have to output the children of some of those ids if any
+            # directories have stopped being directories.
+            for file_id in current_ids:
+                # Examine file_id
+                if discarded_changes:
+                    result = discarded_changes.get(file_id)
+                    old_entry = None
+                else:
+                    result = None
+                if result is None:
+                    old_entry = self._get_entry(self.source, file_id)
+                    new_entry = self._get_entry(self.target, file_id)
+                    result, changes = self._changes_from_entries(
+                        old_entry, new_entry)
+                else:
+                    changes = True
+                # Get this parents parent to examine.
+                new_parent_id = result[4][1]
+                precise_file_ids.add(new_parent_id)
+                if changes:
+                    if (result[6][0] == 'directory' and
+                        result[6][1] != 'directory'):
+                        # This stopped being a directory, the old children have
+                        # to be included.
+                        if old_entry is None:
+                            # Reusing a discarded change.
+                            old_entry = self._get_entry(self.source, file_id)
+                        for child in old_entry.children.values():
+                            precise_file_ids.add(child.file_id)
+                    changed_file_ids.add(result[0])
+                    yield result
 
 
 class MultiWalker(object):

=== added file 'doc/developers/content-filtering.txt'
--- doc/developers/content-filtering.txt	1970-01-01 00:00:00 +0000
+++ doc/developers/content-filtering.txt	2009-08-26 03:20:32 +0000
@@ -0,0 +1,147 @@
+*****************
+Content Filtering
+*****************
+
+Content filtering is the feature by which Bazaar can do line-ending
+conversion or keyword expansion so that the files that appear in the
+working tree are not precisely the same as the files stored in the
+repository.
+
+This document describes the implementation; see the user guide for how to
+use it.
+
+
+We distinguish between the *canonical form* which is stored in the
+repository and the *convenient form* which is stored in the working tree.
+The convenient form will for example use OS-local newline conventions or
+have keywords expanded, and the canonical form will not.  We use these
+names rather than eg "filtered" and "unfiltered" because filters are
+applied when both reading and writing so those names might cause
+confusion.
+
+Content filtering is only active on working trees that support it, which
+is format 2a and later.
+
+Content filtering is configured by rules that match file patterns.
+
+Filters
+*******
+
+Filters come in pairs: a read filter (reading convenient->canonical) and
+a write filter.  There is no requirement that they be symmetric or that
+they be deterministic from the input, though in general both these
+properties will be true.  Filters are allowed to change the size of the
+content, and things like line-ending conversion commonly will.
+
+Filters are fed a sequence of byte chunks (so that they don't have to
+hold the whole file in memory).  There is no guarantee that the chunks
+will be aligned with line endings.  Write filters are passed a context
+object through which they can obtain some information about eg which
+file they're working on.  (See ``bzrlib.filters`` docstring.)
+
+These are at the moment strictly *content* filters: they can't make
+changes to the tree like changing the execute bit, file types, or
+adding/removing entries.
+
+Conventions
+***********
+
+bzrlib interfaces that aren't explicitly specified to deal with the
+convenient form should return the canonical form.  Whenever we have the
+SHA1 hash of a file, it's the hash of the canonical form.
+
+
+Dirstate interactions
+*********************
+
+The dirstate file should store, in the column for the working copy, the cached
+hash and size of the canonical form, and the packed stat fingerprint for
+which that cache is valid.  This implies that the stored size will
+in general be different to the size in the packed stat.  (However, it
+may not always do this correctly - see
+<https://bugs.edge.launchpad.net/bzr/+bug/418439>.)
+
+The dirstate is given a SHA1Provider instance by its tree.  This class
+can calculate the (canonical) hash and size given a filename.  This
+provides a hook by which the working tree can make sure that when the
+dirstate needs to get the hash of the file, it takes the filters into
+account.
+
+
+User interface
+**************
+
+Most commands that deal with the text of files present the
+canonical form.  Some have options to choose.
+
+
+Performance considerations
+**************************
+
+Content filters can have serious performance implications.  For example,
+getting the size of (the canonical form of) a file is easy and fast when
+there are no content filters: we simply stat it.  However, when there
+are filters that might change the size of the file, determining the
+length of the canonical form requires reading in and filtering the whole
+file.
+
+Formats from 1.14 onwards support content filtering, so having fast
+paths for the case where content filtering is not possible is not
+generally worthwhile.  In fact, they're probably harmful by causing
+extra edges in test coverage and performance.
+
+We need to have things be fast even when filters are in use and then
+possibly do a bit less work when there are no filters configured.
+
+
+Future ideas and open issues
+****************************
+
+* We might benefit from having filters declare some of their properties
+  statically, for example that they're deterministic or can round-trip
+  or won't change the length of the file.  However, common cases like
+  crlf conversion are not guaranteed to round-trip and may change the
+  length, so perhaps adding separate cases will just complicate the code
+  and tests.  So overall this does not seem worthwhile.
+
+* In a future workingtree format, it might be better not to separately
+  store the working-copy hash and size, but rather just a stat fingerprint
+  at which point it was known to have the same canonical form as the 
+  basis tree.
+
+* It may be worthwhile to have a virtual Tree-like object that does
+  filtering, so there's a clean separation of filtering from the on-disk
+  state and the meaning of any object is clear.  This would have some
+  risk of bugs where either code holds the wrong object, or their state
+  becomes inconsistent.
+
+  This would be useful in allowing you to get a filtered view of a
+  historical tree, eg to export it or diff it.  At the moment export
+  needs to have its own code to do the filtering.
+
+  The convenient-form tree would talk to disk, and the convenient-form
+  tree would sit on top of that and be used by most other bzr code.
+
+  If we do this, we'd need to handle the fact that the on-disk tree,
+  which generally deals with all of the IO and generally works entirely
+  in convenient form, would also need to be told the canonical hash to
+  store in the dirstate.  This can perhaps be handled by the
+  SHA1Provider or a similar hook.
+
+* Content filtering at the moment is a bit specific to on-disk trees:
+  for instance ``SHA1Provider`` goes directly to disk, but it seems like
+  this is not necessary.
+
+
+See also
+********
+
+* http://bazaar-vcs.org/LineEndings
+
+* http://bazaar-vcs.org/LineEndings/Roadmap
+
+* `Developer Documentation <index.html>`_
+
+* ``bzrlib.filters``
+
+.. vim: ft=rst tw=72

=== modified file 'doc/developers/index.txt'
--- doc/developers/index.txt	2009-08-12 08:03:11 +0000
+++ doc/developers/index.txt	2009-08-26 03:20:32 +0000
@@ -126,6 +126,8 @@
 * `Computing last_modified values <last-modified.html>`_ for inventory
   entries
 
+* `Content filtering <content-filtering.html>`_
+
 * `LCA Tree Merging <lca_tree_merging.html>`_ |--| Merging tree-shape when
   there is not a single unique ancestor (criss-cross merge).
 

=== modified file 'doc/developers/inventory.txt'
--- doc/developers/inventory.txt	2009-07-14 23:23:36 +0000
+++ doc/developers/inventory.txt	2009-08-26 03:20:32 +0000
@@ -585,6 +585,29 @@
 
 Currently we attempt to expand/interpret the request so that the user is not
 required to understand all the internal constraints of the system: if they
-request 'foo/bar' we automatically include foo. This works to an extent but on
-review we are creating inconsistent deltas by the way we do this. We need to
-avoid all the known causes of inconsistency in our delta creation logic.
+request 'foo/bar' we automatically include foo. This works but can surprise
+the user sometimes when things they didn't explicitly request are committed.
+
+Different trees can use different algorithms to expand the request as long as
+they produce consistent deltas. As part of getting a consistent UI we require
+that all trees expand the paths requested downwards. Beyond that as long as 
+the delta is consistent it is up to the tree.
+
+Given two trees, source and target, and a set of selected file ids to check for
+changes and if changed in a delta between them, we have to expand that set by
+the following rules, to get consistent deltas. The test for consistency is that
+if the resulting delta is applied to source, to create a third tree 'output',
+and the paths in the delta match the paths in source and output, only one file
+id is at each path in output, and no file ids are missing parents, then the 
+delta is consistent.
+
+Firstly, the parent ids to the root for all of the file ids that have actually
+changed must be considered. Unless they are all examined the paths in the delta
+may be wrong.
+
+Secondly, when an item included in the delta has a new path which is the same
+as a path in source, the fileid of that path in source must be included.
+Failing to do this leads to multiple ids tryin to share a path in output.
+
+Thirdly, when an item changes its kind from 'directory' to anything else in the
+delta, all of the direct children of the directory in source must be included.

=== modified file 'doc/developers/releasing.txt'
--- doc/developers/releasing.txt	2009-08-14 06:27:37 +0000
+++ doc/developers/releasing.txt	2009-08-26 03:20:32 +0000
@@ -10,6 +10,13 @@
 .. contents::
 
 
+Preconditions
+-------------
+
+#. Download the pqm plugin and install it into your ``~/.bazaar/plugins``::
+
+     bzr branch lp:bzr-pqm ~/.bazaar/plugins/pqm
+
 Starting the release phase
 --------------------------
 
@@ -34,10 +41,6 @@
 
      bzr branch trunk prepare-1.14
 
-#. Download the pqm plugin and install it into your ``~/.bazaar/plugins``::
-
-     bzr branch lp:bzr-pqm ~/.bazaar/plugins/pqm
-
 #. Configure pqm-submit for this branch, with a section like this in
    ``~/.bazaar/locations.conf``::
 
@@ -63,9 +66,8 @@
      ./tools/check-newsbugs.py NEWS
 
    (But note there can be some false positives, and this script may be
-   flaky 
-   <https://bugs.edge.launchpad.net/bzr/+bug/354985>.  Don't let this
-   slow you down too much.)
+   flaky <https://bugs.edge.launchpad.net/bzr/+bug/354985>.  Don't let
+   this slow you down too much.)
 
 #. Summarize into one or two paragraphs what's new in this release.
 

